import click
from .api_wrapper import request_data
from .error_msg import error_msg


@click.command(help="Get Scan Status")
@click.argument('Scan_id')
def status(scan_id):
    try:
        data = request_data('GET', '/scans/'+str(scan_id) + '/latest-status')
        print("\nLast Status update : " + data['status'] + "\n")
    except Exception as E:
        error_msg(E)



import click
from .api_wrapper import request_delete
from .api_wrapper import request_data


@click.command(help="Delete an Object by it's ID")
@click.argument('tid')
@click.option('-scan', is_flag=True, help='Delete a Scan by Scan ID')
@click.option('-agroup', is_flag=True, help='Delete an access group by access group ID')
@click.option('-tgroup', is_flag=True, help='Delete a target-group by target-group ID')
@click.option('-policy', is_flag=True, help='Delete a Policy by Policy ID')
@click.option('-asset', is_flag=True, help='Delete an Asset by Asset UUID')
@click.option('-container', is_flag=True, help='Delete a container by \'/repository/image/tag\'')
@click.option('-repository', is_flag=True, help='Delete Repository from Container Security')
@click.option('-value', is_flag=True, help="Delete a Tag by Value UUID")
@click.option('-category', is_flag=True, help="Delete a Tag Category by UUID")
@click.option('--c', default='', help="Delete a tag by Category / Value pair. Requires --v")
@click.option('--v', default='', help="Delete a tag by Category / Value pair. Requires --c")
@click.option('--bytag', default='', help="Delete assets by Tag. Ex: OS:Linux -- navi delete Linux --bytag tag.OS")
def delete(tid, scan, agroup, tgroup, policy, asset, container, repository, value, category, bytag, c, v):

    if bytag != '':
        print("\nI'm deleting all of the assets associated with your Tag\n")
        payload = {'query': {'field': str(bytag), 'operator': 'set-has', 'value': str(tid)}}
        request_data('POST', '/api/v2/assets/bulk-jobs/delete', payload=payload)

    if c != '':
        if v == '':
            print("value is required.  Please use --v option when deleting tab by value pair")
            exit()
        else:
            tagdata = request_data('GET', '/tags/values')
            for tags in tagdata['values']:
                if c == tags['category_name']:
                    if v == tags['value']:
                        value_uuid = tags['uuid']
                        request_delete('DELETE', '/tags/values/' + str(value_uuid))

    if scan:
        print("\nI'm deleting your Scan Now")
        request_delete('DELETE', '/scans/' + str(tid))

    if agroup:
        print("\nI'm deleting your Access Group Now")
        request_delete('DELETE', ('/access-groups/' + str(tid)))

    if tgroup:
        print("\nI'm deleting your Target group Now")
        request_delete('DELETE', ('/target-groups/' + str(tid)))

    if policy:
        print("\nI'm deleting your Policy Now")
        request_delete('DELETE', ('/policies/' + str(tid)))

    if asset:
        print("\nI'm deleting your asset Now")
        request_delete('DELETE', '/workbenches/assets/' + str(tid))

    if container:
        print("\nI'm deleting your container")
        request_delete('DELETE', '/container-security/api/v2/images/' + str(tid))

    if value:
        print("\nI'm deleting your Tag Value")
        request_delete('DELETE', '/tags/values/' + str(tid))

    if category:
        print("\nI'm Deleting your Category")
        request_delete('delete', '/tags/categories/' + str(tid))

    if repository:
        print("\nI'm Deleting your Repository")
        request_delete('delete', '/container-security/api/v2/' + str(repository))



import click
import getpass
from .database import new_db_connection
from .dbconfig import create_keys_table


@click.command(help="Enter or Reset your Keys")
@click.option("-clear", is_flag=True, help="Show my Keys on the screen")
def keys(clear):
    create_keys_table()
    # assumption is that the user keys didn't work or don't exist
    print("Hey you don't have any Keys!")
    if clear:
        access_key = input("Please provide your Access Key : ")
        secret_key = input("Please provide your Secret Key : ")
    else:
        access_key = getpass.getpass("Please provide your Access Key : ")
        secret_key = getpass.getpass("Please provide your Secret Key : ")

    key_dict = (access_key, secret_key)

    database = r"navi.db"
    conn = new_db_connection(database)
    with conn:
        sql = '''INSERT or IGNORE into keys(access_key, secret_key) VALUES(?,?)'''
        cur = conn.cursor()
        cur.execute(sql, key_dict)



import click, time
from .database import new_db_connection
from .api_wrapper import request_data
from .send_mail import send_email


def grab_smtp():
    # grab SMTP information
    database = r"navi.db"
    conn = new_db_connection(database)
    with conn:
        cur = conn.cursor()
        cur.execute("SELECT * from smtp;")
        rows = cur.fetchall()

        for row in rows:

            server = row[0]
            port = row[1]
            from_email = row[2]
            password = row[3]

        return server, port, from_email, password


@click.command(help="Mail yourself a Report")
@click.option('-latest', is_flag=True, help='Email Vulnerability Summary Information')
@click.option('-consec', is_flag=True, help="Email Container Security Summary Information")
@click.option('-webapp', is_flag=True, help="Email Web Application Scanning Summary Information")
def mail(latest, consec, webapp):
    try:
        # grab SMTP information
        server, port, from_email, password = grab_smtp()
        to_email = input("Please enter the email you wish send this mail to: ")
        subject = input("Please enter the Subject of the email : ")

        subject += " - Emailed by navi Pro"

        # start the message with the proper heading
        msg = "\r\n".join([
            "From: {}".format(from_email),
            "To: {}".format(to_email),
            "Subject: {}".format(subject),
            "", ])

        if latest:
            data = request_data('GET', '/scans')
            time_list = []
            e = {}
            for x in range(len(data["scans"])):
                # keep UUID and Time together
                # get last modication date for duration computation
                epoch_time = data["scans"][x]["last_modification_date"]
                # get the scanner ID to display the name of the scanner
                d = data["scans"][x]["id"]
                # need to identify type to compare against pvs and agent scans
                scan_type = str(data["scans"][x]["type"])
                # don't capture the PVS or Agent data in latest
                while scan_type not in ['pvs', 'agent', 'webapp', 'lce']:
                    # put scans in a list to find the latest
                    time_list.append(epoch_time)
                    # put the time and id into a dictionary
                    e[epoch_time] = d
                    break

            # find the latest time
            grab_time = max(time_list)

            # get the scan with the corresponding ID
            grab_uuid = e[grab_time]

            # turn epoch time into something readable
            epock_latest = time.strftime("%a, %d %b %Y %H:%M:%S +0000", time.localtime(grab_time))
            msg += "\nThe last Scan run was at : {}\n".format(epock_latest)

            # pull the scan data
            details = request_data('GET', '/scans/' + str(grab_uuid))

            scanner_name = details["info"]['scanner_name']
            name = details["info"]["name"]
            hostcount = details["info"]["hostcount"]

            msg += "\nThe Scanner name is : {}"\
                "\nThe Name of the scan is {}\n"\
                "The {} host(s) that were scanned are below :\n".format(scanner_name, name, hostcount)

            for x in range(len(details["hosts"])):
                hostname = details["hosts"][x]["hostname"]
                msg += "\n {}".format(hostname)

            start = time.strftime("%a, %d %b %Y %H:%M:%S ", time.localtime(details["info"]["scan_start"]))

            msg += "\n\nScan start : {}".format(start)

            try:
                stop = time.strftime("%a, %d %b %Y %H:%M:%S ", time.localtime(details["info"]["scan_end"]))
                msg += "Scan finish : {}".format(stop)

                duration = (details["info"]["scan_end"] - details["info"]["scan_start"]) / 60
                msg += "Duration : {} Minutes".format(duration)
            except:
                print("This scan is still running")

            msg += "\nScan Notes Below : \n\n"

            for x in range(len(details["notes"])):
                title = details["notes"][x]["title"]
                message = details["notes"][x]["message"]
                msg += "{} \n {}".format(title, message)

            msg += "\n\n"

        if consec:
            consec_data = request_data('GET', '/container-security/api/v2/images?limit=1000')
            msg += "\n\nContainer Name - Repo Name - Tag - Docker ID - # of Vulns\n---------------------------------------\n"

            for images in consec_data["items"]:
                name = images["name"]
                docker_id = str(images["imageHash"])
                vulns = str(images["numberOfVulns"])
                repo = str(images["repoName"])
                tag = str(images["tag"])
                msg += "{} : {} : {} : {} : {}\n".format(name, repo, tag, docker_id, vulns)

        if webapp:
            webapp_data = request_data('GET', '/scans')

            # cycle through all of the scans and pull out the webapp scan IDs
            msg += "\n\n Web Application Scan Summary\n-----------------------------------------\n"
            for scans in webapp_data['scans']:

                if scans['type'] == 'webapp':
                    scan_details = request_data('GET', '/scans/' + str(scans['id']))
                    try:
                        hostname = scan_details['hosts'][0]['hostname']
                    except KeyError:
                        hostname = " "
                    try:
                        message = scan_details['notes'][0]['message']
                    except KeyError:
                        message = " "
                    try:
                        critical = scan_details['hosts'][0]['critical']
                    except KeyError:
                        critical = 0
                    try:
                        high = scan_details['hosts'][0]['high']
                    except KeyError:
                        high = 0
                    try:
                        medium = scan_details['hosts'][0]['medium']
                    except KeyError:
                        medium = 0
                    try:
                        low = scan_details['hosts'][0]['low']
                    except KeyError:
                        low = 0

                    if message != "Job expired while pending status.":
                        msg += "\nFQDN : {}\n" \
                               "Scan Message: " \
                               "{}\n\n" \
                               "Vulnerability Summary\n----------------------\n" \
                               "Critical : {}\n" \
                               "High {}\n" \
                               "Medium {}\n" \
                               "Low {}\n".format(hostname, message, critical, high, medium, low)

        print("Here is a copy of your email that was Sent")
        print(msg)
        send_email(from_email, to_email, msg, server, password, port)
    except Exception as E:
        print("Your Email information may be incorrect")
        print("Run the 'SMTP' command to correct your information")
        print(E)



import click
from .licensed_export import licensed_export
from .agent_export import agent_export
from .webapp_export import webapp_export
from .consec_export import consec_export
from .csv_export import csv_export
from .lumin_export import lumin_export
from .database import new_db_connection
from .tag_export import tag_export
from .tag_helper import tag_checker
from .network_export import network_export
from .query_export import query_export
from .was_v2_export import was_export


@click.command(help="Export data into a CSV")
@click.option('-assets', is_flag=True, help='Exports all Asset data into a CSV')
@click.option('-agents', is_flag=True, help="Export all Agent data into a CSV")
@click.option('-webapp', is_flag=True, help="Export Webapp Scan Summary into a CSV")
@click.option('-was', is_flag=True, help="Export Webapp Scan Summary into a CSV - WAS V2")
@click.option('-consec', is_flag=True, help="Export Container Security Summary into a CSV")
@click.option('-licensed', is_flag=True, help="Export a List of all the Licensed Assets")
@click.option('-lumin', is_flag=True, help="Export all Asset data including ACR and AES into a CSV. This will take some time")
@click.option('--network', default='', help="Export All assets of a given network")
@click.option('--query', default='', help="Export assets by query to the vuln db")
@click.option('-bytag', is_flag=True, help="Export all assets by tag; Include ACR and AES into a CSV")
@click.option('--c', default='', help="Export bytag with the following Category name")
@click.option('--v', default='', help="Export bytag with the Tag Value; requires --c and Category Name")
@click.option('--ec', default='', help="Exclude tag from export with Tag Category; requires --ev")
@click.option('--ev', default='', help="Exclude tag from export with Tag Value; requires --ec")
def export(assets, agents, webapp, consec, licensed, lumin, network, query, bytag, c, v, ec, ev, was):
    if assets:
        print("\nExporting your data now. Saving asset_data.csv now...\n")
        csv_export()

    if agents:
        print("\nExporting your data now. Saving agent_data.csv now...\n")
        agent_export()

    if webapp:
        print("\nExporting your data now. Saving webapp_data.csv now...\n")
        webapp_export()

    if consec:
        print("\nExporting your data now. Saving consec_data.csv now...\n")
        consec_export()

    if licensed:
        print("\nExporting your data now. Saving licensed_data.csv now...\n")
        licensed_export()

    if lumin:
        print("\nExporting your data now. This could take some time.  300 Assets per minute max.")
        print("Saving asset_lumin.csv now...\n")
        lumin_export()

    if network:
        print("\nExporting your data now. Saving network_data.csv now...")
        network_export(network)

    if query != '':
        query_export(query)

    if bytag:
        if c == '':
            print("Tag Category is required.  Please use the --c command")
            exit()

        if v == '':
            print("Tag Value is required. Please use the --v command")
            exit()

        database = r"navi.db"
        conn = new_db_connection(database)
        with conn:
            try:
                new_list = []
                cur = conn.cursor()
                cur.execute("SELECT asset_uuid, asset_ip from tags where tag_key='" + c + "' and tag_value='" + v + "';")

                assets = cur.fetchall()

                for asset in assets:
                    # This will need to change to UUID once the API gets fixed for Lumin; right not it is by IP
                    # for Each IP check to see if it exists in the exclude tag pair.  If it doesn't add it to the list.
                    check_for_no = tag_checker(asset[1], ec, ev)
                    if check_for_no == 'no':
                        new_list.append(asset[0])
            except conn.OperationalError:
                print('Sorry Right now, navi doesn\'t support \' in a tag')

        tag_export(new_list)

    if was:
        print("\nExporting your data now. Saving was_v2_data.csv now...")
        was_export()



from flask import Flask, render_template, request
import sqlite3
from sqlite3 import Error
from navi.plugins.api_wrapper import request_data
import time

# locale.setlocale(locale.LC_ALL, 'en_US')

starttime = time.time()

app = Flask(__name__)


def new_db_connection(db_file):
    # create a connection to our database
    conn = None
    try:
        # A database file will be created if one doesn't exist
        conn = sqlite3.connect(db_file, timeout=30.0)
    except Error as E:
        print(E)
    return conn


@app.route('/')
def get_info():
    try:
        database = r"navi.db"
        conn = new_db_connection(database)
        with conn:
            cur = conn.cursor()
            cur.execute("SELECT * FROM assets;")

            rows = cur.fetchall()

            return render_template('index.html', rows=rows)
    except Error as e:
        print(e)


@app.route('/sqlquery/', methods=["POST"])
def get_info_by_query():
    database = r"navi.db"
    conn = new_db_connection(database)
    query = request.form['sqlstmt']
    with conn:
        cur = conn.cursor()
        cur.execute(query)

        data = cur.fetchall()

        query_info = []
        for info in data:
            query_info.append(info)

    return render_template('sqlquery.html', query_info=query_info)


@app.route('/containers/', methods=["GET"])
def get_container_info():
    cs_info = []
    data = request_data('GET', '/container-security/api/v2/images?limit=1000')

    for images in data["items"]:
        image_list = [images["name"], images["repoName"], images["tag"], images["imageHash"], images["numberOfVulns"]]
        cs_info.append(image_list)

    return render_template('containers.html', cs_info=cs_info)


if __name__ == '__main__':
    app.run(host="0.0.0.0", port=8000)



from .database import new_db_connection, create_table


def create_keys_table():
    # Create Tables
    database = r"navi.db"
    key_conn = new_db_connection(database)
    key_table = """CREATE TABLE IF NOT EXISTS keys (
                            access_key text,
                            secret_key text
                            );"""
    key_conn.execute('pragma journal_mode=wal;')
    create_table(key_conn, key_table)


def create_vulns_table():
    database = r"navi.db"
    vuln_conn = new_db_connection(database)
    vuln_table = """CREATE TABLE IF NOT EXISTS vulns (
                            navi_id integer PRIMARY KEY,
                            asset_ip text, 
                            asset_uuid text, 
                            asset_hostname text, 
                            first_found text, 
                            last_found text, 
                            output text, 
                            plugin_id text, 
                            plugin_name text, 
                            plugin_family text, 
                            port text, 
                            protocol text, 
                            severity text, 
                            scan_completed text, 
                            scan_started text, 
                            scan_uuid text, 
                            schedule_id text, 
                            state text
                            );"""
    vuln_conn.execute('pragma journal_mode=wal;')
    create_table(vuln_conn, vuln_table)


def create_assets_table():
    database = r"navi.db"
    asset_conn = new_db_connection(database)
    create_asset_table = """CREATE TABLE IF NOT EXISTS assets (
                            ip_address text,
                            hostname text,
                            fqdn text,
                            uuid text PRIMARY KEY,
                            first_found text,
                            last_found text, 
                            operating_system text,
                            mac_address text, 
                            agent_uuid text,
                            last_licensed_scan_date text,
                            network text
                            );"""
    asset_conn.execute('pragma journal_mode=wal;')
    create_table(asset_conn, create_asset_table)


def create_tag_table():
    database = r"navi.db"
    tag_conn = new_db_connection(database)
    create_tags_table = """CREATE TABLE IF NOT EXISTS tags (
                        tag_id integer PRIMARY KEY,
                        asset_uuid text,
                        asset_ip,
                        tag_key text,
                        tag_uuid text,
                        tag_value text,
                        tag_added_date text
                        );"""
    tag_conn.execute('pragma journal_mode=wal;')
    create_table(tag_conn, create_tags_table)



from .ip import ip
from .find import find
from .report import report
from .list import display
from .pause import pause
from .start import start
from .stop import stop
from .resume import resume
from .mac import mac
from .delete import delete
from .status import status
from .api import api
from .export import export
from .lumin import lumin
from .update import update
from .add import add
from .scan import scan
from .http import http
from .listen import listen
from .keys import keys
from .smtp import smtp
from .mail import mail
from .new_tag import tag
from .agroup import agroup
from .navi_portal import portal
from .was import was
from .change import change
from .network import network


def plugin_loader(group):
    group.add_command(ip)
    group.add_command(find)
    group.add_command(report)
    group.add_command(display)
    group.add_command(pause)
    group.add_command(resume)
    group.add_command(start)
    group.add_command(stop)
    group.add_command(mac)
    group.add_command(delete)
    group.add_command(status)
    group.add_command(api)
    group.add_command(export)
    group.add_command(lumin)
    group.add_command(update)
    group.add_command(add)
    group.add_command(scan)
    group.add_command(http)
    group.add_command(listen)
    group.add_command(keys)
    group.add_command(smtp)
    group.add_command(mail)
    group.add_command(tag)
    group.add_command(agroup)
    group.add_command(portal)
    group.add_command(was)
    group.add_command(change)
    group.add_command(network)



import click
from .th_asset_export import asset_export
from .th_vuln_export import vuln_export
from sqlite3 import Error


@click.command(help="Update local repository")
@click.option('-assets', is_flag=True, help="Update the Agent data")
@click.option('-vulns', is_flag=True, help="Update the Vulnerability Data data")
@click.option('--days', default='30', help="Limit the download to X # of days")
@click.option('--exid', default='0', help="Download using a specified Export ID")
@click.option('--threads', default=10, help="Control the threads to speed up or slow down downloads - (1-10)")
def update(assets, vulns, days, exid, threads):
    # Limit the amount of threads to avoid issues
    if threads != 10:
        print("\nUsing {} thread(s) at your request".format(threads))
        if threads not in range(1, 11):
            print("Enter a value between 1 and 10")
            exit()
    try:
        if exid == ' ':
            exid = '0'

        if assets:
            asset_export(days, exid, threads)

        if vulns:
            vuln_export(days, exid, threads)

        if not assets and not vulns:
            vuln_export(days, exid, threads)

            # default for assets should be 90 days to support licensing exports
            asset_export(90, exid, threads)
    except Error as E:
        print("\n Have you entered your keys?\n")
        print("Error: ", E, "\n")



import click
from .database import new_db_connection
from .api_wrapper import request_data


def check_agroup_exists(aname):
    agroups = request_data('GET', '/access-groups')
    rvalue = 'no'
    for group in agroups['access_groups']:
        if str(group['name']).lower() == str(aname).lower():
            rvalue = group['id']
    return rvalue


@click.command(help="Create an Access group Based on a Tag or Agent Group")
@click.option('--name', default='', help="Create an Access group with the following Name")
@click.option('-tag', is_flag=True, help="Create a Access Group by a Tag")
@click.option('--c', default='', help="Category name to use: requires --v and Value Name")
@click.option('--v', default='', help="Tag Value to use; requires --c and Category Name")
@click.option('--group', default='', help="Create a Access Group based on a Agent Group")
@click.option('--user', default='', help="User you want to Assign to the Access Group")
@click.option('--usergroup', default='', help="User Group you want to assign to the Access Group")
@click.option('-scan', is_flag=True, help="Set Scan ONLY permission")
@click.option('-view', is_flag=True, help="Set View ONLY permission")
@click.option('-scanview', is_flag=True, help="Set Scan AND View permissions")
def agroup(name, tag, c, v, group, user, usergroup, scan, view, scanview):
    new_list = []
    permission = []
    choice = 'none'
    permtype = 'none'

    if user == '' and usergroup == '':
        print("You Need to use '--user' or '--usergroup' command and supply a user or group. e.g: user@yourdomain or Linux Admins")
        exit()

    if user != '':
        permtype = 'user'
        choice = user

    if usergroup != '':
        permtype = 'group'
        choice = usergroup

    if name == '':
        print("You need to use the --name command to name your Access Group")
        exit()

    if tag:
        if c == '':
            print("Tag Category is required.  Please use the --c command")
            exit()

        if v == '':
            print("Tag Value is required. Please use the --v command")
            exit()

        database = r"navi.db"
        conn = new_db_connection(database)
        with conn:
            cur = conn.cursor()
            cur.execute("SELECT asset_uuid, asset_ip from tags where tag_key='" + c + "' and tag_value='" + v + "';")
            assets = cur.fetchall()
            for asset in assets:
                new_list.append(asset[1])

    if group:
        print()
        print("*" * 50)
        print("API limits agent groups to 5000")
        print("*" * 50)
        querystring = {"limit": "5000"}
        group_data = request_data('GET', '/scanners/1/agent-groups')
        for agent_group in group_data['groups']:
            group_name = agent_group['name']
            group_id = agent_group['id']

            if group_name == group:
                data = request_data('GET', '/scanners/1/agent-groups/' + str(group_id) + '/agents', params=querystring)

                for agent in data['agents']:
                    ip_address = agent['ip']
                    new_list.append(ip_address)

    if scanview:
        permission = ["CAN_VIEW", "CAN_SCAN"]

    if view:
        permission = ["CAN_VIEW"]

    if scan:
        permission = ["CAN_SCAN"]

    if not scan and not view and not scanview:
        print("\nYou must supply a permission")
        print("\nUse one of the following:\n-scan\n-view\n-scanview\n")
        exit()

    payload = {"name": str(name), "access_group_type": "MANAGE_ASSETS", "all-users": True, "rules": [{"type": "ipv4", "operator": "eq", "terms": new_list}],
               "principals": [{"permissions": permission, "type": permtype, "principal_name": choice}]}

    answer = check_agroup_exists(str(name))

    try:
        # Check to see if the list has any IPs in it.
        if new_list:
            if answer == 'no':
                new_access_group = request_data('POST', '/v2/access-groups', payload=payload)
                print("\nYour Access group {} is being created now \n".format(new_access_group['name']))
                print("The UUID is {} \n".format(new_access_group['id']))
            else:
                update_group = request_data('PUT', '/v2/access-groups/' + str(answer), payload=payload)
                print("\nYour Access group {} is being updated now \n".format(update_group['name']))
                print("The UUID is {} \n".format(update_group['id']))
        else:
            print("\nYour list was empty so nothing happened\n")
    except TypeError:
        print("\nAccess group? - Check the Username")



import click
from .api_wrapper import request_data


@click.command(help="Change network options")
@click.option("--net", default='00000000-0000-0000-0000-000000000000', help="Select Network ID")
@click.option("--age", default='', help="Adjust Asset Age-out policy for a given network ID")
def network(net, age):
    network_data = request_data('GET', '/networks/' + net)
    name = network_data['name']

    payload = {"assets_ttl_days": age, "description": "TTL adjusted by navi", "name": name}
    if age != '':
        data = request_data('PUT', '/networks/' + net, payload=payload)
        print(data)



import click
from sqlite3 import Error
import pprint
from .api_wrapper import request_data
from .database import new_db_connection


def find_by_plugin(plugin):
    try:
        database = r"navi.db"
        conn = new_db_connection(database)
        with conn:
            cur = conn.cursor()
            cur.execute("SELECT asset_ip, asset_uuid, output from vulns where plugin_id=%s;" % plugin)

            rows = cur.fetchall()

            for row in rows:
                print("\nIP Address: " + row[0])
                print("UUID : " + row[1])
                print("\n--- Plugin " + plugin + " Output ---\n")
                print(row[2])
                print("--- End plugin Output ---")
    except Error as e:
        print(e)


@click.command(help="Find Containers, Web Apps, Credential failures, Ghost Assets")
@click.option('--plugin', default='', help='Find Assets where this plugin fired')
@click.option('-docker', is_flag=True, help="Find Running Docker Containers")
@click.option('-webapp', is_flag=True, help="Find Web Servers running")
@click.option('-creds', is_flag=True, help="Find Credential failures")
@click.option('--scantime', default='', help='Find Assets where the scan duration is over X mins')
@click.option('-ghost', is_flag=True, help='Find Assets that were discovered by a AWS Connector but not scanned')
@click.option('--port', default='', help='Find assets with a open port provided')
@click.option('--query', default='', help='Query the db directly and display the output')
@click.option('--output', default='', help='Find Assets based on the text in the output. Requires --plugin"')
@click.option('--name', default='', help="Find Assets based on Text in a plugin Name")
def find(plugin, docker, webapp, creds, scantime, ghost, port, query, output, name):

    if output != '' and plugin == '':
        print("You must supply a plugin")
        exit()

    if plugin != '':
        if not str.isdigit(plugin):
            print("You didn't enter a number")
            exit()
        else:
            try:
                plugin_db = r"navi.db"
                plugin_conn = new_db_connection(plugin_db)
                with plugin_conn:
                    plugin_cur = plugin_conn.cursor()
                    # See if we want to refine our search by the output found in this plugin
                    # this needs to have a JOIN statement to reduce the amount of IPs
                    if output != "":
                        plugin_cur.execute("SELECT asset_ip, asset_uuid, output from vulns where plugin_id='" + plugin + "' and output LIKE '%" + output + "%';")
                    else:
                        find_by_plugin(plugin)

                    plugin_data = plugin_cur.fetchall()
                    for x in plugin_data:
                        print(x[0])
                        print('*' * 20)
                        print(x[2])
                        print('*' * 150)
                        print('*' * 150)
            except Error:
                pass

    if docker:
        print("Searching for RUNNING docker containers...")
        find_by_plugin(str(93561))

    if webapp:
        database = r"navi.db"
        conn = new_db_connection(database)
        with conn:
            cur = conn.cursor()
            cur.execute("SELECT * from vulns where plugin_id='1442';")

            data = cur.fetchall()
            print("\nWeb Servers found by plugin 1442")
            print("-------------------------------")
            for plugins in data:
                web = plugins[6]  # output
                wsplit = web.split("\n")

                server = wsplit[1]
                web_port = plugins[10]  # port number
                proto = plugins[11]  # Protocol
                asset = plugins[1]  # Ip address
                print()
                print(server, "is running on: ", web_port, "/", proto, "On :", asset)
        try:
            print("\n\nWeb Servers/SSH Servers found by plugin '22964';")
            print("-------------------------------\n")
            conn2 = new_db_connection(database)
            with conn2:
                cur2 = conn.cursor()
                cur2.execute("SELECT output, port, asset_ip from vulns where plugin_id='22964'")
                data = cur2.fetchall()

                for plugins in data:
                    print("\n", plugins[0][:-1], plugins[1], "On :", plugins[2])
                print()
        except IndexError:
            print("No information for plugin 22964")

    if creds:
        print("I'm looking for credential issues...Please hang tight")
        find_by_plugin(str(104410))

    if scantime != '':
        database = r"navi.db"
        conn = new_db_connection(database)
        with conn:
            print("\n*** Below are the assets that took longer than " + str(scantime) + " minutes to scan ***")
            cur = conn.cursor()
            cur.execute("SELECT * from vulns where plugin_id='19506';")

            data = cur.fetchall()
            try:
                print("\nAsset IP".ljust(16), "Asset UUID".ljust(40), "Started".ljust(25), "Finished".ljust(25), "Scan UUID")
                print("-" * 150)
                for vulns in data:

                    output = vulns[6]

                    # split the output by return
                    parsed_output = output.split("\n")

                    # grab the length so we can grab the seconds
                    length = len(parsed_output)

                    # grab the scan duration- second to the last variable
                    duration = parsed_output[length - 2]

                    # Split at the colon to grab the numerical value
                    seconds = duration.split(" : ")

                    # split to remove "secs"
                    number = seconds[1].split(" ")

                    # grab the number for our minute calculation
                    final_number = number[0]

                    # convert seconds into minutes
                    minutes = int(final_number) / 60

                    # grab assets that match the criteria
                    if minutes > int(scantime):
                        try:
                            print(str(vulns[1]).ljust(15), str(vulns[2]).ljust(40), str(vulns[14]).ljust(25), str(vulns[13]).ljust(25), str(vulns[15]))
                        except ValueError:
                            pass
                print()
            except ValueError:
                pass

    if ghost:
        try:

            ghost_query = {"date_range": "30", "filter.0.filter": "sources", "filter.0.quality": "set-hasonly", "filter.0.value": "AWS"}
            data = request_data('GET', '/workbenches/assets', params=ghost_query)
            print("\nSource".ljust(11), "IP".ljust(15), "FQDN".ljust(45), "First seen")
            print("-" * 150)
            for assets in data['assets']:
                for source in assets['sources']:
                    if source['name'] == 'AWS':
                        aws_ip = assets['ipv4'][0]
                        try:
                            aws_fqdn = assets['fqdn'][0]
                        except IndexError:
                            aws_fqdn = assets['fqdn'][0]

                        print(str(source['name']).ljust(10), str(aws_ip).ljust(15), str(aws_fqdn).ljust(45), source['first_seen'])
            print()

            gcp_query = {"date_range": "30", "filter.0.filter": "sources", "filter.0.quality": "set-hasonly", "filter.0.value": "GCP"}
            gcp_data = request_data('GET', '/workbenches/assets', params=gcp_query)
            for gcp_assets in gcp_data['assets']:
                for gcp_source in gcp_assets['sources']:
                    if gcp_source['name'] == 'GCP':
                        gcp_ip = gcp_assets['ipv4'][0]
                        try:
                            gcp_fqdn = gcp_assets['fqdn'][0]
                        except IndexError:
                            gcp_fqdn = "NO FQDN FOUND"
                            
                        print(gcp_source['name'].ljust(10), gcp_ip.ljust(15), gcp_fqdn.ljust(45), gcp_source['first_seen'])
            print()

            az_query = {"date_range": "30", "filter.0.filter": "sources", "filter.0.quality": "set-hasonly", "filter.0.value": "AZURE"}
            az_data = request_data('GET', '/workbenches/assets', params=az_query)
            for az_assets in az_data['assets']:
                for az_source in az_assets['sources']:
                    if az_source['name'] == 'AZURE':

                        az_ip = az_assets['ipv4'][0]
                        try:
                            az_fqdn = az_assets['fqdn'][0]
                        except IndexError:
                            az_fqdn = "NO FQDN Found"

                        print(az_source['name'].ljust(10), az_ip.ljust(15), az_fqdn.ljust(45), az_source['first_seen'])
            print()
                
        except Exception as E:
            print("Check your API keys or your internet connection")
            print(E)

    if port != '':
        database = r"navi.db"
        conn = new_db_connection(database)
        with conn:
            cur = conn.cursor()
            cur.execute("SELECT * from vulns where port=" + port + " and (plugin_id='11219' or plugin_id='14272' or plugin_id='14274' or plugin_id='34220' or plugin_id='10335');")

            data = cur.fetchall()

            try:
                print("\nThe Following assets had Open ports found by various plugins")
                print("\nIP address".ljust(20), " UUID".ljust(45), " Plugins")
                print("-" * 80)
                for vulns in data:
                    print(vulns[1].ljust(20), str(vulns[2]).ljust(45), vulns[7])
            except ValueError:
                pass

    if query != '':
        database = r"navi.db"
        query_conn = new_db_connection(database)
        with query_conn:

            cur = query_conn.cursor()
            cur.execute(query)

            data = cur.fetchall()
            pprint.pprint(data)

    if name != '':
        try:
            database = r"navi.db"
            conn = new_db_connection(database)
            with conn:
                cur = conn.cursor()
                cur.execute("SELECT asset_ip, asset_uuid, plugin_name, plugin_id from vulns where plugin_name LIKE '%" + name + "%';")

                plugin_data = cur.fetchall()
                print("\nThe Following assets had '{}' in the Plugin Name".format(name))
                print("\nIP address".ljust(20), " UUID".ljust(45), " Plugin Name".ljust(65), " Plugin ID")
                print("-" * 150)
                for vulns in plugin_data:
                    print(vulns[0].ljust(20), str(vulns[1]).ljust(45), str(vulns[2]).ljust(65), vulns[3])
                print()
        except Error:
            pass



import click
from flask import Flask, render_template, request
import sqlite3
from sqlite3 import Error
from .api_wrapper import request_data


@click.command(help="A web interface to explore the navi DB [BETA- Doesn't work in Docker]")
def portal():
    app = Flask(__name__)

    def new_db_connection(db_file):
        # create a connection to our database
        conn = None
        try:
            # A database file will be created if one doesn't exist
            conn = sqlite3.connect(db_file, timeout=30.0)
        except Error as E:
            print(E)
        return conn

    @app.route('/')
    def get_info():
        try:
            database = r"navi.db"
            conn = new_db_connection(database)
            with conn:
                cur = conn.cursor()
                cur.execute("SELECT * FROM assets;")

                rows = cur.fetchall()

                return render_template('index.html', rows=rows)
        except Error as e:
            print(e)

    @app.route('/sqlquery/', methods=["GET", "POST"])
    def get_info_by_query():
        database = r"navi.db"
        conn = new_db_connection(database)
        query = request.form['sqlstmt']
        print(query)
        with conn:
            cur = conn.cursor()
            cur.execute(query)

            data = cur.fetchall()

            query_info = []
            for info in data:
                query_info.append(info)

        return render_template('sqlquery.html', query_info=query_info)

    @app.route('/containers/', methods=["GET"])
    def get_container_info():
        cs_info = []
        data = request_data('GET', '/container-security/api/v2/images?limit=1000')

        for images in data["items"]:
            image_list = [images["name"], images["repoName"], images["tag"], images["imageHash"],
                          images["numberOfVulns"]]
            cs_info.append(image_list)

        return render_template('containers.html', cs_info=cs_info)

    app.run(host="0.0.0.0", port=8000)




import click
from .scanners import nessus_scanners
from .api_wrapper import request_data
from .error_msg import error_msg


@click.command(help="Quickly Scan a Target")
@click.argument('targets')
def scan(targets):
    try:
        print("\nChoose your Scan Template")
        print("1.   Basic Network Scan")
        print("2.   Discovery Scan")
        print("3.   Web App Overview")
        print("4.   Web App Scan")
        print("5.   WAS SSL SCAN")
        option = input("Please enter option #.... ")
        if option == '1':
            template = "731a8e52-3ea6-a291-ec0a-d2ff0619c19d7bd788d6be818b65"
        elif option == '2':
            template = "bbd4f805-3966-d464-b2d1-0079eb89d69708c3a05ec2812bcf"
        elif option == '3':
            template = "58323412-d521-9482-2224-bdf5e2d65e6a4c67d33d4322677f"
        elif option == '4':
            template = "09805055-a034-4088-8986-aac5e1c57d5f0d44f09d736969bf"
        elif len(option) == 52:
            template = str(option)
        else:
            print("Using Basic scan since you can't follow directions")
            template = "731a8e52-3ea6-a291-ec0a-d2ff0619c19d7bd788d6be818b65"

        print("Here are the available scanners")
        print("Remember, don't pick a Cloud scanner for an internal IP address")
        print("Remember also, don't chose a Webapp scanner for an IP address")
        nessus_scanners()
        scanner_id = input("What scanner do you want to scan with ?.... ")

        print("creating your scan of : " + targets + "  Now...")

        payload = dict(uuid=template, settings={"name": "navi-Pro Created Scan of " + targets,
                                                "enabled": "True",
                                                "scanner_id": scanner_id,
                                                "text_targets": targets})

        # create a new scan
        data = request_data('POST', '/scans', payload=payload)

        # pull scan ID after Creation
        scan_id = str(data["scan"]["id"])

        # launch Scan
        request_data('POST', '/scans/' + scan_id + '/launch')

        print("I started your scan, your scan ID is: ", scan_id)

    except Exception as E:
        error_msg(E)



import sqlite3
from sqlite3 import Error


def new_db_connection(db_file):
    # create a connection to our database
    conn = None
    try:
        # A database file will be created if one doesn't exist
        conn = sqlite3.connect(db_file, timeout=60.0)
    except Error as E:
        print(E)
    return conn


def create_table(conn, table_information):
    try:
        c = conn.cursor()
        c.execute('pragma journal_mode=wal;')
        c.execute(table_information)
    except Error as e:
        print(e)


def insert_assets(conn, assets):
    sql = '''INSERT or IGNORE into assets(ip_address, hostname, fqdn, uuid, first_found, last_found, operating_system,
                       mac_address, agent_uuid, last_licensed_scan_date, network) VALUES(?,?,?,?,?,?,?,?,?,?,?)'''
    cur = conn.cursor()
    cur.execute('pragma journal_mode=wal;')
    cur.execute(sql, assets)


def insert_tags(conn, tags):
    sql = '''INSERT or IGNORE into tags(tag_id, asset_uuid, asset_ip, tag_key, tag_uuid, tag_value, tag_added_date) VALUES(?,?,?,?,?,?,?)'''
    cur = conn.cursor()
    cur.execute('pragma journal_mode=wal;')
    cur.execute(sql, tags)


def drop_tables(conn, table):
    try:
        drop_table = '''DROP TABLE {}'''.format(table)
        cur = conn.cursor()
        cur.execute('pragma journal_mode=wal;')
        cur.execute(drop_table)
    except Error:
        pass


def insert_vulns(conn, vulns):
    sql = '''INSERT or IGNORE into vulns(
                            navi_id,
                            asset_ip, 
                            asset_uuid, 
                            asset_hostname, 
                            first_found, 
                            last_found, 
                            output, 
                            plugin_id, 
                            plugin_name, 
                            plugin_family, 
                            port, 
                            protocol, 
                            severity, 
                            scan_completed, 
                            scan_started, 
                            scan_uuid, 
                            schedule_id, 
                            state
    ) VALUES(?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?)'''

    cur = conn.cursor()
    cur.execute('pragma journal_mode=wal;')
    cur.execute(sql, vulns)



import requests
from json import JSONDecodeError
from .database import new_db_connection


def grab_headers():
    database = r"navi.db"
    conn = new_db_connection(database)
    with conn:
        cur = conn.cursor()
        cur.execute("SELECT * from keys;")
        rows = cur.fetchall()
        for row in rows:
            access_key = row[0]
            secret_key = row[1]
    return {'Content-type': 'application/json', 'user-agent': 'navi-5.2.0', 'X-ApiKeys': 'accessKey=' + access_key + ';secretKey=' + secret_key}


def request_delete(method, url_mod):

    # set the Base URL
    url = "https://cloud.tenable.com"

    try:
        r = requests.request(method, url + url_mod, headers=grab_headers(), verify=True)
        if r.status_code == 200:
            print("Success!\n")
        elif r.status_code == 404:
            print('\nCheck your query...', r)
        elif r.status_code == 429:
            print("\nToo many requests at a time...\n", r)
        else:
            print("Something went wrong...Don't be trying to hack me now", r)
    except ConnectionError:
        print("Check your connection...You got a connection error")


def request_data(method, url_mod, **kwargs):

    # set the Base URL
    url = "https://cloud.tenable.com"

    # check for params and set to None if not found
    try:
        params = kwargs['params']
    except KeyError:
        params = None

    # check for a payload and set to None if not found
    try:
        payload = kwargs['payload']
    except KeyError:
        payload = None

    # Retry the download three times
    for x in range(1, 3):
        try:
            r = requests.request(method, url + url_mod, headers=grab_headers(), params=params, json=payload, verify=True)
            if r.status_code == 200:
                return r.json()

            if r.status_code == 202:
                # This response is for some successful posts.
                print("\nSuccess!\n")
                break
            elif r.status_code == 404:
                print('\nCheck your query...I can\'t find what you\'re looking for', r)
                return r.json()
            elif r.status_code == 429:
                print("\nToo many requests at a time...\n", r)
                break
            elif r.status_code == 400:
                print("\nThe object you tried to create may already exist\n")
                print("If you are changing scan ownership, there is a bug where 'empty' scans won't be moved")
                break
            elif r.status_code == 403:
                print("\nYou are not authorized! You need to be an admin\n", r)
                break
            elif r.status_code == 409:
                print("\nScan is not Active\n", r)
                break
            elif r.status_code == 504:
                print("\nOne of the Threads and an issue during download...Retrying...\n", r)
                break
            else:
                print("Something went wrong...Don't be trying to hack me now", r)
                break
        except ConnectionError:
            print("Check your connection...You got a connection error. Retying")
            continue
        except JSONDecodeError:
            print("Download Error. Retrying")
            continue



import csv
from .database import new_db_connection
from .api_wrapper import request_data


def tag_export(tag_list):
    print("This export can take some time.  ~300 assets per minute.")
    database = r"navi.db"
    conn = new_db_connection(database)
    with conn:

        # Create our headers - We will Add these two our list in order
        header_list = ["IP Address", "Hostname", "FQDN", "UUID", "First Found", "Last Found", "Operating System",
                       "Mac Address", "Agent-UUID", "last Licensed Scan Date", 'Network','Info', 'Low', 'Medium', 'High', 'Critical', 'Asset Exposure Score', 'Asset Criticality Score']
        cur = conn.cursor()
        cur.execute("SELECT * from assets;")

        data = cur.fetchall()

        # Crete a csv file object
        with open('bytag_lumin.csv', mode='w') as csv_file:
            agent_writer = csv.writer(csv_file, delimiter=',', quotechar='"')

            # write our Header information first
            agent_writer.writerow(header_list)

            # Loop through each asset
            for assets in data:
                export_list = []
                if assets[3] in tag_list:
                    for atr in assets:
                        # Cycle through the Database and populate the new list
                        export_list.append(atr)

                    asset_id = assets[3]  # Grab the UUID to make API calls

                    try:
                        asset_info = request_data('GET', '/workbenches/assets/' + asset_id + '/info')

                        for vuln in asset_info['info']['counts']['vulnerabilities']['severities']:
                            export_list.append(vuln["count"])  # Add the vuln counts to the new list

                        try:
                            export_list.append(asset_info['info']['exposure_score'])  # add the exposure score
                            export_list.append(asset_info['info']['acr_score'])  # add the ACR
                        except KeyError:  # Non Lumin users may not have these values
                            pass

                    except ConnectionError:
                        print("Check your API keys or your internet connection")
                    # write to the CSV
                    agent_writer.writerow(export_list)
        print("Export success! - bytag_lumin.csv")



import click
import time
from .api_wrapper import request_data
from .scan_details import scan_details
from .error_msg import error_msg
from .database import new_db_connection
import textwrap


@click.command(help="Get the Latest Scan information")
@click.option('-latest', is_flag=True, help="Report the Last Scan Details")
@click.option('--container', default='', help='Report CVSS 7 or above by \'/repository/image/tag\'')
@click.option('--docker', default='', help='Report CVSS 7 or above by Docker ID')
@click.option('--comply', default='', help='Check to see if your container complies with your Corporate Policy')
@click.option('--details', default='', help='Report Scan Details including Vulnerability Counts by Scan ID')
@click.option('--summary', default='', help="Report Scan Summary information by Scan ID")
@click.option('--network', default='', help="Report assets of a given network")
def report(latest, container, docker, comply, details, summary, network):
    # get the latest Scan Details
    if latest:
        try:
            data = request_data('GET', '/scans')
            time_list = []
            e = {}
            for x in data["scans"]:
                # keep UUID and Time together
                # get last modication date for duration computation
                epoch_time = x["last_modification_date"]
                # get the scanner ID to display the name of the scanner
                d = x["id"]
                # need to identify type to compare against pvs and agent scans
                scan_type = str(x["type"])
                # don't capture the PVS or Agent data in latest
                while scan_type not in ['pvs', 'agent', 'webapp', 'lce']:
                    # put scans in a list to find the latest
                    time_list.append(epoch_time)
                    # put the time and id into a dictionary
                    e[epoch_time] = d
                    break

            # find the latest time
            grab_time = max(time_list)

            # get the scan with the corresponding ID
            grab_uuid = e[grab_time]

            # turn epoch time into something readable
            epock_latest = time.strftime("%a, %d %b %Y %H:%M:%S +0000", time.localtime(grab_time))
            print("\nThe last Scan run was at " + epock_latest)
            scan_details(str(grab_uuid))
        except Exception as E:
            error_msg(E)

    if container:
        try:
            data = request_data('GET', '/container-security/api/v2/reports' + str(container))
            try:
                for vulns in data['findings']:
                    if float(vulns['nvdFinding']['cvss_score']) >= 7:
                        print("CVE ID :", vulns['nvdFinding']['cve'])
                        print("CVSS Score : ", vulns['nvdFinding']['cvss_score'])
                        print("----------------------")
                        print("\nDescription : \n\n", vulns['nvdFinding']['description'])
                        print("\nRemediation : \n\n", vulns['nvdFinding']['remediation'])
                        print("----------------------END-------------------------\n")
            except TypeError:
                print("This Container has no data or is not found")
            except ValueError:
                pass
        except Exception as E:
            error_msg(E)

    if docker:
        try:
            data = request_data('GET', '/container-security/api/v1/reports/by_image?image_id=' + str(docker))

            try:
                for vulns in data['findings']:
                    if float(vulns['nvdFinding']['cvss_score']) >= 7:
                        print("CVE ID :", vulns['nvdFinding']['cve'])
                        print("CVSS Score : ", vulns['nvdFinding']['cvss_score'])
                        print("-----------------------")
                        print("\nDescription \n\n: ", vulns['nvdFinding']['description'])
                        print("\nRemediation : \n\n", vulns['nvdFinding']['remediation'])
                        print("----------------------END-------------------------\n")
            except TypeError:
                print("This Container has no data or is not found")
            except ValueError:
                pass
        except Exception as E:
            error_msg(E)

    if comply:
        try:
            data = request_data('GET', '/container-security/api/v1/policycompliance?image_id=' + str(comply))

            print("Status : ", data['status'])
        except Exception as E:
            error_msg(E)

    if details:
        try:
            data = request_data('GET', '/scans/' + str(details))
            try:
                print()
                print("Scan Details for Scan ID : " + details)
                print()
                print("Notes: \b")
                try:
                    print(data['notes'][0]['message'])
                except IndexError:
                    pass
                print()
                print("Vulnerability Counts")
                print("--------------------")
                print("Critical : ", data['hosts'][0]['critical'])
                print("high : ", data['hosts'][0]['high'])
                print("medium : ", data['hosts'][0]['medium'])
                print("low : ", data['hosts'][0]['low'])
                try:
                    print("--------------")
                    print("Score : ", data['hosts'][0]['score'])
                except IndexError:
                    pass
                print()
                print("Vulnerability Details".ljust(80), "Vuln Count")
                print("-" * 100)

                for vulns in data['vulnerabilities']:
                    if vulns['severity'] != 0:
                        print(textwrap.shorten(str(vulns['plugin_name']), 80).ljust(80), vulns['count'])
            except TypeError:
                print("Check the scan ID")
        except Exception as E:
            error_msg(E)

    if summary:
        try:
            print("\nHere is the Summary of your Scan :")
            print("----------------------------------")
            scan_details(str(summary))
        except Exception as E:
            error_msg(E)

    if network:
        database = r"navi.db"

        conn = new_db_connection(database)
        with conn:
            cur = conn.cursor()
            cur.execute("SELECT ip_address, fqdn, last_licensed_scan_date from assets where network == '" + network + "';")
            data = cur.fetchall()

            print("IP Address".ljust(15), "Full Qualified Domain Name".ljust(65), "Licensed Scan Date")
            print("-".ljust(91, "-"))
            print()
            for asset in data:
                ipv4 = asset[0]
                fqdn = asset[1]
                licensed_date = asset[2]
                print(str(ipv4).ljust(15), str(fqdn).ljust(65), licensed_date)
        print()



import click
import time
from .api_wrapper import request_data
from .scanners import nessus_scanners
from .database import new_db_connection
from .error_msg import error_msg
from .licensed_count import get_licensed
from sqlite3 import Error
import textwrap


@click.command(help="Display or Print information found in Tenable.io")
@click.option('-scanners', is_flag=True, help="List all of the Scanners")
@click.option('-users', is_flag=True, help="List all of the Users")
@click.option('-exclusions', is_flag=True, help="List all Exclusions")
@click.option('-containers', is_flag=True, help="List all containers and their Vulnerability  Scores")
@click.option('-logs', is_flag=True, help="List The actor and the action in the log file")
@click.option('-running', is_flag=True, help="List the running Scans")
@click.option('-scans', is_flag=True, help="List all Scans")
@click.option('-nnm', is_flag=True, help="Nessus Network Monitor assets and their vulnerability scores")
@click.option('-assets', is_flag=True, help="Assets found in the last 30 days")
@click.option('-policies', is_flag=True, help="Scan Policies")
@click.option('-connectors', is_flag=True, help="List Connector Details and Status")
@click.option('-agroup', is_flag=True, help="List Access Groups and Status")
@click.option('-status', is_flag=True, help="Print T.io Status and Account info")
@click.option('-agents', is_flag=True, help="Print Agent information")
@click.option('-webapp', is_flag=True, help='Print Web App Scans')
@click.option('-tgroup', is_flag=True, help='Print Target Groups')
@click.option('-licensed', is_flag=True, help='Print License information')
@click.option('-tags', is_flag=True, help='Print Tag Categories and values')
@click.option('-categories', is_flag=True, help='Print all of the Tag Categories and their UUIDs')
@click.option('-smtp', is_flag=True, help="Print your smtp information")
@click.option('-cloud', is_flag=True, help="Print Cloud assets found in the last 30 days by the connectors")
@click.option('-networks', is_flag=True, help="Print Network IDs")
@click.option('-version', is_flag=True, help="Display current version of Navi")
def display(scanners, users, exclusions, containers, logs, running, scans, nnm, assets, policies, connectors, agroup, status, agents, webapp, tgroup, licensed, tags, categories, smtp, cloud, networks, version):

    if scanners:
        nessus_scanners()

    if users:
        try:
            data = request_data('GET', '/users')
            print("\nUser Name".ljust(35), "Login email".ljust(40), "User UUID")
            print("-" * 100)
            for user in data["users"]:
                print(str(user["name"]).ljust(34), str(user["username"]).ljust(40), user['uuid'])
            print()

        except Exception as E:
            error_msg(E)

    if exclusions:
        try:
            data = request_data('GET', '/exclusions')
            for x in data["exclusions"]:
                print("\nExclusion Name : ", x["name"], '\n')
                print(x["members"], '\n')

        except KeyError:
            print("No Exclusions Set, or there could be an issue with your API keys")

    if containers:
        try:
            data = request_data('GET', '/container-security/api/v2/images?limit=1000')
            print("\nContainer Name".ljust(35), "Repository ID".ljust(35), "Tag".ljust(15), "Docker ID".ljust(15), "# of Vulns".ljust(10))
            print("-" * 125)
            try:
                for images in data["items"]:
                    print(str(images["name"]).ljust(35), str(images["repoName"]).ljust(35), str(images["tag"]).ljust(15), str(images["imageHash"]).ljust(15), str(images["numberOfVulns"]).ljust(25))
            except KeyError:
                pass
            print()
        except Exception as E:
            error_msg(E)

    if logs:
        try:
            data = request_data('GET', '/audit-log/v1/events')
            print("\nEvent Date".ljust(24), "Action Taken".ljust(30), "User".ljust(30))
            print("-" * 100)
            for log in data['events']:
                received = log['received']
                action = log['action']
                actor = log['actor']['name']
                print(str(received).ljust(24), str(action).ljust(30), str(actor))

        except Exception as E:
            error_msg(E)
        print()

    if running:
        try:
            data = request_data('GET', '/scans')
            run = 0
            print("\nScan Name".ljust(61), "Scan ID".ljust(10), "Status".ljust(30))
            print("-" * 100)
            for scan in data['scans']:
                if scan['status'] == "running":
                    run = run + 1
                    name = scan['name']
                    scan_id = scan['id']
                    current_status = scan['status']

                    print(str(name).ljust(60), str(scan_id).ljust(10), str(current_status))
            print()

            if run == 0:
                print("No running scans")
        except Exception as E:
            error_msg(E)

    if scans:
        print()
        print("*" * 40)
        print("This endpoint is extremely limited. 1 to 2 calls per min")
        print("If you get a 429 error, wait a minute and try again")
        print("*" * 40)
        try:
            data = request_data('GET', '/scans')
            print("\nScan Name".ljust(61), "Scan ID".ljust(10), "Status".ljust(30))
            print("-" * 100)
            for scan in data['scans']:
                name = scan['name']
                scan_id = scan['id']
                scan_status = scan['status']
                print(str(name).ljust(60), str(scan_id).ljust(10), str(scan_status))
            print()

        except Exception as E:
            error_msg(E)

    if nnm:
        try:
            # dynamically find the PVS sensor
            nnm_data = request_data('GET', '/scans')

            for scan in nnm_data["scans"]:

                if str(scan["type"]) == "pvs":
                    nnm_id = scan["id"]
                    try:
                        data = request_data('GET', '/scans/' + str(nnm_id) + '/')
                        if len(data["hosts"]) != 0:
                            print("Here are the assets and their scores last found by Nessus Network Monitor")
                            print("   IP Address     : Score")
                            print("----------------")

                            for host in data["hosts"]:
                                print(str(host["hostname"]) + " :  " + str(host["score"]))
                            print()
                    except KeyError:
                        pass
        except Exception as E:
            error_msg(E)

    if assets:
        try:
            data = request_data('GET', '/workbenches/assets/?date_range=30') #, params=asset_limit)

            print("\nBelow are the assets found in the last 30 days")
            print("\nIP Address(es)".ljust(35), "FQDN(s)".ljust(65), "Exposure Score".ljust(15), "Sources")
            print("-" * 120)
            for asset in data["assets"]:
                sources = []
                for source in asset["sources"]:
                    sources.append(source['name'])

                print(str(asset["ipv4"]).ljust(36), str(asset["fqdn"]).ljust(64), str(asset["exposure_score"]).ljust(15), sources)

        except Exception as E:
            error_msg(E)

    if policies:
        try:
            data = request_data('GET', '/policies')
            print("\nPolicy Name".ljust(41), "Description".ljust(61), "Template ID")
            print("-" * 100)
            for policy in data['policies']:
                print(str(policy['name']).ljust(40), str(policy['description']).ljust(60), policy['template_uuid'] )
            print()
        except Exception as E:
            error_msg(E)

    if connectors:
        try:
            data = request_data('GET', '/settings/connectors')
            print("\nType".ljust(11), "Connector Name".ljust(40), "Connnector ID".ljust(40), "Last Sync".ljust(30), "Schedule ")
            print("-" * 150)
            for conn in data["connectors"]:
                schedule = str(conn['schedule']['value']) + " " + str(conn['schedule']['units'])
                try:
                    last_sync = conn['last_sync_time']

                except KeyError:
                    last_sync = "Hasn't synced"
                print(str(conn['type']).ljust(10), str(conn['name']).ljust(40), str(conn['id']).ljust(40), last_sync.ljust(30), schedule)
        except Exception as E:
            error_msg(E)
        print()

    if agroup:
        try:
            data = request_data('GET', '/access-groups')
            print("\nGroup Name".ljust(26), "Group ID".ljust(40), "Last Updated".ljust(25), "Rules")
            print("-" * 120)
            for group in data["access_groups"]:
                try:
                    updated = group['updated_at']
                except KeyError:
                    updated = "Not Updated"
                details = request_data('GET', '/access-groups/'+str(group['id']))

                try:
                    for rule in details['rules']:
                        rules = str(rule['terms'])
                except KeyError:
                    rules = "Not Rule based"
                print(str(group['name']).ljust(25), str(group['id']).ljust(40), str(updated).ljust(25), textwrap.shorten(rules, width=60))
            print()
        except Exception as E:
            error_msg(E)

    if status:
        try:
            data = request_data('GET', "/server/properties")
            session_data = request_data('GET', "/session")

            print("\nTenable IO Information")
            print("-----------------------")
            print("Container ID : ", session_data["container_id"])
            print("Container UUID :", session_data["container_uuid"])
            print("Container Name : ", session_data["container_name"])
            print("Site ID :", data["analytics"]["site_id"])
            print("Region : ", data["region"])

            print("\nLicense information")
            print("--------------------")
            print("Licensed Assets : ", get_licensed())
            print("Agents Used : ", data["license"]["agents"])
            print("Expiration Date : ", data["license"]["expiration_date"])
            print("Scanners Used : ", data["license"]["scanners"])
            print("Users : ", data["license"]["users"])
            print("\nEnabled Apps")
            print("---------")
            print()
            for key in data["license"]["apps"]:
                print(key)
                print("-----")
                try:
                    print("Expiration: ", data["license"]["apps"][key]["expiration_date"])

                except KeyError:
                    pass
                print("Mode: ", data["license"]["apps"][key]["mode"])
                print()

        except Exception as E:
            error_msg(E)

    if agents:
        querystring = {"limit": "5000"}
        try:
            data = request_data('GET', '/scanners/1/agents', params=querystring)
            print("\nAgent Name".ljust(46), "IP Address".ljust(20), "Last Connect Time".ljust(20), "Last Scanned Time".ljust(20), "Status".ljust(10), "Groups")
            print("-" * 140)
            for agent in data['agents']:
                last_connect = agent['last_connect']
                last_connect_time = time.strftime("%b %d %H:%M:%S", time.localtime(last_connect))

                try:
                    last_scanned = agent['last_scanned']
                    last_scanned_time = time.strftime("%b %d %H:%M:%S", time.localtime(last_scanned))
                except KeyError:
                    # I assume if we can't pull as scanned time, it doesn't exist
                    last_scanned_time = "Not Scanned"
                groups = ''
                try:
                    for group in agent['groups']:
                        groups = groups + ", " + group['name']
                except KeyError:
                    pass
                print(str(agent['name']).ljust(45), str(agent['ip']).ljust(20), str(last_connect_time).ljust(20),
                      str(last_scanned_time).ljust(20), str(agent['status']).ljust(10), str(groups[1:]))
        except Exception as E:
            error_msg(E)
        print()

    if webapp:
        print()
        print("*" * 40)
        print("This is using the V1 and Was 1.0 endpoints")
        print("use the 'navi was --help' to see what is available for v2")
        print("*" * 40)
        try:
            data = request_data('GET', '/scans')
            print("\nScan Name".ljust(61), "Scan ID".ljust(10), "Status".ljust(30))
            print("-" * 100)
            for scan in data['scans']:
                if scan['type'] == 'webapp':
                    name = scan['name']
                    scan_id = scan['id']
                    scan_status = scan['status']
                    print(str(name).ljust(60), str(scan_id).ljust(10), str(scan_status))
            print()

        except Exception as E:
            error_msg(E)

    if tgroup:
        print()
        print("*" * 40)
        print("Target Groups are going to be retired use the Migration script to covert them to tags")
        print("https://github.com/packetchaos/tio_automation/blob/master/migrate_target_groups.py")
        print("*" * 40)
        data = request_data('GET', '/target-groups')
        print("\nTarge Group Name".ljust(41), "TG ID".ljust(10), "Owner".ljust(30), "Members")
        print("-" * 100)
        try:
            for targets in data['target_groups']:
                mem = targets['members']
                print(str(targets['name']).ljust(40), str(targets['id']).ljust(10), str(targets['owner']).ljust(30), textwrap.shorten(mem, width=60))
            print()
        except Exception as E:
            error_msg(E)

    if licensed:
        print("\nLicensed Count : ", get_licensed())
        print()
        database = r"navi.db"
        conn = new_db_connection(database)
        with conn:
            cur = conn.cursor()
            cur.execute("SELECT ip_address, fqdn, last_licensed_scan_date from assets where last_licensed_scan_date !=' ';")
            data = cur.fetchall()

            print("IP Address".ljust(20), "Full Qualified Domain Name".ljust(65), "Licensed Date")
            print("-" * 120)
            print()
            for asset in data:
                ipv4 = asset[0]
                fqdn = asset[1]
                licensed_date = asset[2]
                # Don't display Web applications in this output
                if ipv4 != " ":
                    print(str(ipv4).ljust(20), str(fqdn).ljust(65), licensed_date)
        print()

    if tags:
        data = request_data('GET', '/tags/values')
        print("\nTags".ljust(33), "Value".ljust(35), "Value UUID")
        print('-'.rjust(92, '-'), "\n")
        for tag_values in data['values']:
            try:
                tag_value = tag_values['value']
                uuid = tag_values['uuid']
            except KeyError:
                tag_value = "Value Not Set Yet"
                uuid = "NO Value set"
            print(str(tag_values['category_name']).rjust(30), ":", str(tag_value).ljust(35), str(uuid).ljust(25))
        print()

    if categories:
        data = request_data('GET', '/tags/categories')
        print("\nTag Categories".ljust(31), "Category UUID")
        print('-'.rjust(50, '-'), "\n")
        for cats in data['categories']:

            category_name = cats['name']
            category_uuid = cats['uuid']

            print(str(category_name).ljust(30), str(category_uuid).ljust(25))
        print()

    if smtp:
        try:
            database = r"navi.db"
            conn = new_db_connection(database)
            with conn:
                cur = conn.cursor()
                cur.execute("SELECT server, port, from_email from smtp;")
                data = cur.fetchall()
                for settings in data:
                    print("\nYour email server: {}".format(settings[0]))
                    print("The email port is: {}".format(settings[1]))
                    print("Your email is: {}\n".format(settings[2]))
        except Error as E:
            print("\nYou have no SMTP information saved.\n")
            print("Error: ", E, "\n")

    if cloud:
        query = {"date_range": "30", "filter.0.filter": "sources", "filter.0.quality": "set-has", "filter.0.value": "AWS",
                 "filter.1.filter": "sources", "filter.1.quality": "set-has", "filter.1.value": "AZURE",
                 "filter.2.filter": "sources", "filter.2.quality": "set-has", "filter.2.value": "GCP", "filter.search_type":"or"}
        data = request_data('GET', '/workbenches/assets', params=query)
        print("\nSource".ljust(11), "IP".ljust(15), "FQDN".ljust(45), "UUID".ljust(40), "First seen")
        print("-" * 150)
        for assets in data['assets']:
            for source in assets['sources']:
                if source['name'] != 'NESSUS_SCAN':
                    asset_ip = assets['ipv4'][0]
                    uuid = assets['id']
                    try:
                        asset_fqdn = assets['fqdn'][0]
                    except IndexError:
                        asset_fqdn = "NO FQDN found"

                    print(source['name'].ljust(10), asset_ip.ljust(15), asset_fqdn.ljust(45),uuid.ljust(40), source['first_seen'])
        print()

    if networks:
        try:
            data = request_data('GET', '/networks')
            print("\nNetwork Name".ljust(26), "# of Scanners".ljust(16), "UUID")
            print("-" * 100)
            for network in data['networks']:
                print(network['name'].ljust(25), str(network['scanner_count']).ljust(15), network['uuid'])
        except Exception as E:
            error_msg(E)
        print()

    if version:
        print("\nNavi Version 5.2.0\n")



import click
from sqlite3 import Error
from .api_wrapper import request_data
from .database import new_db_connection


def plugin_by_ip(ipaddr, plugin):
    try:
        database = r"navi.db"
        conn = new_db_connection(database)
        with conn:
            try:
                cur = conn.cursor()
                cur.execute("SELECT output from vulns where asset_ip=\"%s\" and plugin_id=%s" % (ipaddr, plugin))
                rows = cur.fetchall()
                print(rows[0][0])
            except:
                pass

    except Error as e:
        print(e)

    except IndexError:
        print("No information found for this plugin")


@click.command(help="Get IP specific information")
@click.argument('ipaddr')
@click.option('--plugin', default='', help='Find Details on a particular plugin ID')
@click.option('-n', is_flag=True, help='Netstat Established(58561) and Listening and Open Ports(14272)')
@click.option('-p', is_flag=True, help='Patch Information - 66334')
@click.option('-t', is_flag=True, help='Trace Route - 10287')
@click.option('-o', is_flag=True, help='Process Information - 70329')
@click.option('-c', is_flag=True, help='Connection Information - 64582')
@click.option('-s', is_flag=True, help='Services Running - 22964')
@click.option('-r', is_flag=True, help='Local Firewall Rules - 56310')
@click.option('-patches', is_flag=True, help='Missing Patches - 38153')
@click.option('-d', is_flag=True, help="Scan Detail: 19506 plugin output")
@click.option('-software', is_flag=True, help="Find software installed on Unix(22869) of windows(20811) hosts")
@click.option('-outbound', is_flag=True, help="outbound connections found by nnm")
@click.option('-exploit', is_flag=True, help="Display exploitable vulnerabilities")
@click.option('-critical', is_flag=True, help="Display critical vulnerabilities")
@click.option('-details', is_flag=True, help="Details on an Asset: IP, UUID, Vulns, etc")
@click.pass_context
def ip(ctx, ipaddr, plugin, n, p, t, o, c, s, r, patches, d, software, outbound, exploit, critical, details):
    plugin_by_ip(ipaddr, plugin)

    if d:
        click.echo('\nScan Detail')
        click.echo('----------------\n')
        plugin_by_ip(ipaddr, str(19506))

    if n:
        click.echo("\nNetstat info")
        click.echo("Established and Listening")
        click.echo("----------------\n")
        plugin_by_ip(ipaddr, str(58651))
        click.echo("\nNetstat Open Ports")
        click.echo("----------------")
        plugin_by_ip(ipaddr, str(14272))

    if p:
        click.echo("\nPatch Information")
        click.echo("----------------\n")
        plugin_by_ip(ipaddr, str(66334))

    if t:
        click.echo("\nTrace Route Info")
        click.echo("----------------\n")
        plugin_by_ip(ipaddr, str(10287))

    if o:
        click.echo("\nProcess Info")
        click.echo("----------------\n")
        plugin_by_ip(ipaddr, str(70329))
        plugin_by_ip(ipaddr, str(110483))

    if patches:
        click.echo("\nMissing Patches")
        click.echo("----------------\n")
        plugin_by_ip(ipaddr, str(38153))
        plugin_by_ip(ipaddr, str(66334))

        click.echo("\nLast Reboot")
        click.echo("----------------\n")
        plugin_by_ip(ipaddr, str(56468))

    if c:
        click.echo("\nConnection info")
        click.echo("----------------\n")
        plugin_by_ip(ipaddr, str(64582))

    if s:
        try:
            database = r"navi.db"
            conn = new_db_connection(database)
            with conn:
                cur = conn.cursor()
                cur.execute("SELECT output, port from vulns where asset_ip=\"%s\" and plugin_id='22964'" % ipaddr)
                data = cur.fetchall()

                for plugins in data:
                    print("\n", plugins[0], plugins[1])
                print()
        except IndexError:
            print("No information for plugin 22964")

    if r:
        click.echo("Local Firewall Info")
        click.echo("----------------")
        plugin_by_ip(ipaddr, str(56310))
        plugin_by_ip(ipaddr, str(61797))

    if software:
        try:
            plugin_by_ip(ipaddr, str(22869))
            plugin_by_ip(ipaddr, str(20811))
        except IndexError:
            print("No Software found")

    if outbound:
        try:
            database = r"navi.db"
            conn = new_db_connection(database)
            with conn:
                cur = conn.cursor()
                cur.execute("SELECT output, port, protocol from vulns where asset_ip=\"%s\" and plugin_id='16'" % ipaddr)

                data = cur.fetchall()
                print("\nIP Address", " - ", "Port", " - ", "Protocol")
                print("-------------------------------")
                for plugins in data:
                    print("\n", plugins[0].ljust(13), plugins[1].ljust(10), plugins[2])
                print()
        except:
            print("No information for plugin 16")

    if exploit:
        try:
            database = r"navi.db"
            conn = new_db_connection(database)
            with conn:
                cur = conn.cursor()
                cur.execute("SELECT uuid from assets where ip_address='" + ipaddr + "';")

                data = cur.fetchall()
                for assets in data:

                    asset_id = assets[0]

                    print("Exploitable Details for : " + ipaddr)
                    print()
                    V = request_data('GET', '/workbenches/assets/' + asset_id + '/vulnerabilities?filter.0.quality=eq&filter.0.filter=plugin.attributes.exploit_available&filter.0.value=True')

                    for plugins in range(len(V['vulnerabilities'])):
                        plugin = V['vulnerabilities'][plugins]['plugin_id']

                        P = request_data('GET', '/plugins/plugin/' + str(plugin))
                        # pprint.pprint(P['attributes'])
                        print("\n----Exploit Info----")
                        print(P['name'])
                        print()
                        for attribute in range(len(P['attributes'])):

                            if P['attributes'][attribute]['attribute_name'] == 'cve':
                                cve = P['attributes'][attribute]['attribute_value']
                                print("CVE ID : " + cve)

                            if P['attributes'][attribute]['attribute_name'] == 'description':
                                description = P['attributes'][attribute]['attribute_value']
                                print("Description")
                                print("------------\n")
                                print(description)
                                print()

                            if P['attributes'][attribute]['attribute_name'] == 'solution':
                                solution = P['attributes'][attribute]['attribute_value']
                                print("\nSolution")
                                print("------------\n")
                                print(solution)
                                print()
        except:
            print("No Exploit Details found for: ", ipaddr)

    if critical:
        try:
            database = r"navi.db"
            conn = new_db_connection(database)
            with conn:
                cur = conn.cursor()
                cur.execute("SELECT uuid from assets where ip_address='" + ipaddr + "';")

                data = cur.fetchall()
                for assets in data:
                    asset_id = assets[0]
                    print("Critical Vulns for Ip Address :" + ipaddr + "\n")
                    vulns = request_data('GET',
                                         "/workbenches/assets/"
                                         + asset_id + "/vulnerabilities?date_range=90")
                    for severities in range(len(vulns["vulnerabilities"])):
                        vuln_name = vulns["vulnerabilities"][severities]["plugin_name"]
                        plugin_id = vulns["vulnerabilities"][severities]["plugin_id"]
                        severity = vulns["vulnerabilities"][severities]["severity"]
                        state = vulns["vulnerabilities"][severities]["vulnerability_state"]

                        # only pull the critical vulns; critical = severity 4
                        if severity >= 4:
                            print("Plugin Name : " + vuln_name)
                            print("ID : " + str(plugin_id))
                            print("Severity : Critical")
                            print("State : " + state)
                            print("----------------\n")
                            plugin_by_ip(str(ipaddr), str(plugin_id))
                            print()
        except:
            print("No Critical Vulnerabilities found for : ", ipaddr)

    if details:
        database = r"navi.db"
        conn = new_db_connection(database)
        with conn:
            cur = conn.cursor()
            cur.execute("SELECT uuid from assets where ip_address='" + ipaddr + "';")

            data = cur.fetchall()
            for assets in data:
                asset_data = request_data('GET', '/workbenches/assets/' + assets[0] + '/info?date_range=90')

                try:
                    asset_id = asset_data['info']['id']

                    print("\nTenable ID")
                    print("--------------")
                    print(asset_id)

                    print("\nNetwork Name")
                    print("--------------")
                    print(asset_data['info']['network_name'])

                    print("\nIdentities")
                    print("--------------")
                    try:
                        for netbioss in asset_data['info']['netbios_name']:
                            print("Netbios - ", netbioss)
                    except KeyError:
                        pass
                    try:
                        for fqdns in asset_data['info']['fqdns']:
                            print("FQDN - ", fqdns)
                    except KeyError:
                        pass

                    try:
                        for hosts in asset_data['info']['hostname']:
                            print("Host Name -", hosts)
                    except KeyError:
                        pass

                    try:
                        for agentname in asset_data['info']['agent_name']:
                            print("Agent Name -", agentname)
                    except KeyError:
                        pass

                    try:
                        for awsid in asset_data['info']['aws_ec2_instance_id']:
                            print("AWS EC2 Instance ID - ", awsid)
                    except KeyError:
                        pass

                    try:
                        for awsamiid in asset_data['info']['aws_ec2_ami_id']:
                            print("AWS EC2 AMI ID - ", awsamiid)
                    except KeyError:
                        pass

                    try:
                        for awsname in asset_data['info']['aws_ec2_name']:
                            print("AWS EC2 Name - ", awsname)
                    except KeyError:
                        pass

                    print("\nOperating Systems")
                    print("--------------")
                    try:
                        for oss in asset_data['info']['operating_system']:
                            print(oss)
                    except KeyError:
                        pass

                    try:
                        print("\nIP Addresses:")
                        print("--------------")
                        for ips in asset_data['info']['ipv4']:
                            print(ips)
                    except KeyError:
                        pass

                    try:
                        print("\nMac Addresses:")
                        print("--------------")
                        for macs in asset_data['info']['mac_address']:
                            print(macs)
                    except KeyError:
                        pass

                    try:
                        print("\nCloud Information:")
                        print("--------------")
                        for zone in asset_data['info']['aws_availability_zone']:
                            print("AWS Availability Zone - ", zone)
                    except KeyError:
                        pass

                    try:
                        for groupname in asset_data['info']['aws_ec2_instance_group_name']:
                            print("AWS Instance group Name - ", groupname)
                    except KeyError:
                        pass

                    try:
                        for zone in asset_data['info']['aws_availability_zone']:
                            print("AWS Availability Zone - ", zone)
                    except KeyError:
                        pass
                    try:
                        for statename in asset_data['info']['aws_ec2_instance_state_name']:
                            print("AWS Instance State - ", statename)
                    except KeyError:
                        pass
                    try:
                        for instatncetype in asset_data['info']['aws_ec2_instance_type']:
                            print("AWS Instance Type - ", instatncetype)
                    except KeyError:
                        pass
                    try:
                        for region in asset_data['info']['aws_region']:
                            print("AWS Region - ", region)
                    except KeyError:
                        pass

                    try:
                        for subnet in asset_data['info']['aws_subnet_id']:
                            print("AWS Subnet ID - ", subnet)
                    except KeyError:
                        pass
                    try:
                        for vpc in asset_data['info']['aws_vpc_id']:
                            print("AWS VPC ID - ", vpc)
                    except KeyError:
                        pass
                    try:
                        for azureid in asset_data['info']['azure_resource_id']:
                            print("Azure Resource ID - ", azureid)
                    except KeyError:
                        pass
                    try:
                        for vmid in asset_data['info']['azure_vm_id']:
                            print("Azure VM ID - ", vmid)
                    except KeyError:
                        pass

                    try:
                        for gcpid in asset_data['info']['gcp_instance_id']:
                            print("GCP Instance ID - ", gcpid)
                    except KeyError:
                        pass
                    try:
                        for projectid in asset_data['info']['gcp_project_id']:
                            print("GCP Project ID- ", projectid)
                    except KeyError:
                        pass
                    try:
                        for gcpzone in asset_data['info']['gcp_zone']:
                            print("GCP Zone - ", gcpzone)
                    except KeyError:
                        pass
                    try:
                        print("\nSources:")
                        print("--------------")
                        for source in asset_data['info']['sources']:
                            print(source['name'])
                    except KeyError:
                        pass
                    try:
                        print("\nTags:")
                        print("--------------")
                        for tags in asset_data['info']['tags']:
                            print(tags["tag_key"], ':', tags['tag_value'])
                    except KeyError:
                        pass

                    try:
                        print("\nVulnerability Counts")
                        print("--------------")
                        asset_info = request_data('GET', '/workbenches/assets/' + asset_id + '/info')

                        for vuln in asset_info['info']['counts']['vulnerabilities']['severities']:
                            print(vuln["name"], " : ", vuln["count"])

                        try:
                            print("\nAsset Exposure Score : ", asset_info['info']['exposure_score'])
                            print("\nAsset Criticality Score :", asset_info['info']['acr_score'])
                        except:
                            pass

                    except:
                        print("Check your API keys or your internet connection")

                    print("\nLast Authenticated Scan Date - ", asset_data['info']['last_authenticated_scan_date'])
                    print("-" * 50)
                    print("-" * 50)
                except:
                    pass



import csv
import time
from .api_wrapper import request_data


def agent_export():
    data = request_data('GET', '/scanners')

    # get US cloud Scanner ID
    for scanner in range(len(data['scanners'])):
        if data['scanners'][scanner]['name'] == 'US Cloud Scanner':
            scan_id = data['scanners'][scanner]['id']
            querystring = {"limit": "5000"}
            # pull agent data from the US cloud Scanner
            agents = request_data('GET', '/scanners/' + str(scan_id) + '/agents', params=querystring)

            with open('agent_data.csv', mode='w') as csv_file:
                agent_writer = csv.writer(csv_file, delimiter=',', quotechar='"')

                header_list = ["Agent Name", "IP Address", "Platform", "Last connected", "Last scanned", "Status"]
                agent_writer.writerow(header_list)

                # cycle through the agents and display the useful information
                for a in agents['agents']:
                    name = a['name']
                    ip = a['ip']
                    platform = a['platform']

                    last_connect = a['last_connect']
                    connect_time = time.strftime("%a, %d %b %Y %H:%M:%S", time.localtime(last_connect))
                    try:
                        last_scanned = a['last_scanned']
                        scanned_time = time.strftime("%a, %d %b %Y %H:%M:%S", time.localtime(last_scanned))
                    except KeyError:
                        scanned_time = "Not Yet Scanned"
                    status = a['status']

                    agent_writer.writerow([name, ip, platform, connect_time, scanned_time, status])
    return



import click
from .api_wrapper import request_data
from .error_msg import error_msg
import time
import uuid
import csv
import textwrap


def web_app_scanners():
    print("\nHere are the available scanners")
    print("Remember, don't pick a Cloud scanner for an internal Site\n")
    scanners = request_data('GET', '/scanners')

    print("Scanner Name".ljust(30), "Scanner UUID")
    print("-" * 50)
    for scanner in scanners['scanners']:

        if str(scanner['supports_webapp']) == 'True':
            print(str(scanner['name']).ljust(29), str(scanner['id']))
    print()


def display_users():
    users = request_data('GET', '/users')
    print("\nUser Name".ljust(40), "User UUID")
    print("-" * 50)
    for user in users['users']:
        print(str(user['user_name'].ljust(39)), str(user['uuid']))
    print()


def create_was_scan(owner_id, temp_id, scanner_id, target, name):
    # use time to ensure UUID is correct
    uuid_offset = int(time.time())

    # create a new WAS UUID
    was_uuid = uuid.uuid1(uuid_offset)

    # Construct the payload
    payload = dict({
        "name": str(name),
        "owner_id": str(owner_id),
        "template_id": str(temp_id),
        "scanner_id": str(scanner_id),
        "settings": {
            "target": str(target)
            }
        }
    )

    request_data('PUT', '/was/v2/configs/' + str(was_uuid), payload=payload)


@click.command(help="Interact with WAS V2 API")
@click.option('-scans', is_flag=True, help="Displays WAS Scans")
@click.option('--start', default='', help="Start Scan with Provided Scan ID")
@click.option('--sd', default='', help="Get Scan Details with Provided Scan ID")
@click.option('--scan', default='', help="Create a scan via FQDN or CSV file name; use -file option for bulk scan creation via CSV file")
@click.option('-file', is_flag=True, help="File name of the CSV containing Web Apps for bulk scan creation")
@click.option('-configs', is_flag=True, help="Show config UUIDs to start or stop scans")
@click.option('--stats', default='', help="Show scan stats")
@click.option('-summary', is_flag=True, help="Summary of all of the Web Apps")
def was(scans, start, sd, scan, file, configs, stats, summary):
    if scans:
        params = {"size": "1000"}
        data = request_data('GET', '/was/v2/scans', params=params)
        print("\nTarget FQDN".ljust(70), "Scan UUID".ljust(40), "Status".ljust(14), "Last Update")
        print("-" * 150)
        try:
            for scan_data in data['data']:
                app_url = scan_data['application_uri']
                was_scan_id = scan_data['scan_id']
                status = scan_data['status']
                updated = scan_data['updated_at']

                print(textwrap.shorten(str(app_url), width=69).ljust(69), str(was_scan_id).ljust(40), str(status).ljust(14), str(updated))
            print()
        except Exception as E:
            error_msg(E)

    if start != '':
        print("\n Your Scan is starting")
        request_data('POST', '/was/v2/configs/' + start + '/scans')

    if sd != '':
        report = request_data('GET', '/was/v2/scans/' + str(sd) + '/report')
        high = []
        meduim = []
        low = []
        name = report['config']['name']
        target = report['config']['settings']['target']
        print()
        print(name)
        print(target)
        print("-" * 40)
        print()
        print("Plugin".ljust(10), "Plugin Name".ljust(60), "Severity".ljust(10), "CVSS3 Base Score".ljust(10))
        print("-" * 100)
        for finding in report['findings']:
            risk = finding['risk_factor']
            plugin_id = finding['plugin_id']
            plugin_name = finding['name']

            if risk == 'high':
                high.append(plugin_id)
            elif risk == 'medium':
                meduim.append(plugin_id)
            elif risk == 'low':
                low.append(plugin_id)

            print(str(plugin_id).ljust(10), str(plugin_name).ljust(60), str(risk).ljust(10), str(finding['cvss3']))

        print("\nSeverity Counts")
        print("-" * 20)
        print("High: ", len(high))
        print("Medium: ", len(meduim))
        print("Low: ", len(low))
        print()

    if scan:
        print("\nChoose your Scan Template")
        print("1.   Web App Overview")
        print("2.   Web App Scan")
        print("3.   WAS SSL SCAN")
        print("4.   WAS Config Scan")
        option = input("Please enter option #.... ")
        if option == '1':
            template = "b223f18e-5a94-4e02-b560-77a4a8246cd3"
        elif option == '2':
            template = "112f3e7f-d83a-4bba-b2c8-df2d22e2fa4c"
        elif option == '3':
            template = "072f4d6b-1dd7-4049-b279-78a56d1c778e"
        elif option == '4':
            template = "3078d0c6-6e81-44de-b585-6921b69ff0ef"
        elif len(option) == 52:
            template = str(option)
        else:
            print("Using Basic scan since you can't follow directions")
            template = "b223f18e-5a94-4e02-b560-77a4a8246cd3"

        # Display scanners
        web_app_scanners()

        # Capture Scanner selection
        scanner_id = input("What scanner do you want to scan with ?.... ")

        # Display Users UUID
        display_users()
        # Capture User UUID selection
        user_uuid = input("Select an Scan owner using the UUID ?.... ")

        scan_name = "navi Created Scan of : " + str(scan)

        if file:
            with open(scan, 'r', newline='') as csv_file:
                web_apps = csv.reader(csv_file)
                for apps in web_apps:
                    for app in apps:
                        scan_name = "navi Created Scan of : " + str(app)
                        create_was_scan(owner_id=user_uuid, scanner_id=scanner_id, name=scan_name, temp_id=template, target=str(app))
                        time.sleep(5)
        else:
            print("\nCreating your scan now for site: " + str(scan))
            create_was_scan(owner_id=user_uuid, scanner_id=scanner_id, name=scan_name, temp_id=template, target=scan)

    if configs:
        params = {"size": "1000"}
        config_info = request_data('GET', '/was/v2/configs', params=params)
        print("Name".ljust(80), "Config ID".ljust(40), "Last Run")
        print("-" * 150)
        for config in config_info['data']:
            try:
                updated = config['last_scan']['updated_at']
            except TypeError:
                updated = "Not Run yet"
            print(textwrap.shorten(str(config['name']), width=80).ljust(80), str(config['config_id']).ljust(40), str(updated))
        print()

    if stats:
        scan_metadata = request_data('GET', '/was/v2/scans/' + str(stats))
        report = request_data('GET', '/was/v2/scans/' + str(stats) + '/report')
        high = []
        medium = []
        low = []
        name = report['config']['name']
        target = report['config']['settings']['target']
        output = ''
        print()
        print(name)
        print(target)
        print("-" * 60)
        print("\nNotes:")
        print("-" * 60)
        for note in report['notes']:
            print(note['title'])
            print("\t- ", note['message'])

        print("\nScan Data Available")
        print("-" * 40)

        try:
            for i in scan_metadata['metadata']:
                print(i, scan_metadata['metadata'][i])
        except TypeError:
            print("None Available")

        for finding in report['findings']:
            risk = finding['risk_factor']
            plugin_id = finding['plugin_id']

            if plugin_id == 98000:
                output = finding['output']

            if risk == 'high':
                high.append(plugin_id)
            elif risk == 'medium':
                medium.append(plugin_id)
            elif risk == 'low':
                low.append(plugin_id)

        print("\nSeverity Counts")
        print("-"*20)
        print("High: ", len(high))
        print("Medium: ", len(medium))
        print("Low: ", len(low))
        print("\nScan Statistics")
        print("-" * 20)
        print(output)
        print()

    if summary:
        # Grab all of the Scans
        params = {"size": "1000"}

        data = request_data('GET', '/was/v2/scans', params=params)
        print("\nScan Name".ljust(38), "Target".ljust(40), "High".ljust(6), "Mid".ljust(6), "Low".ljust(6), "Scan Started".ljust(25), "Scan Finished".ljust(20))
        print("-" * 150)
        for scan_data in data['data']:
            was_scan_id = scan_data['scan_id']
            status = scan_data['status']
            start = scan_data['started_at']
            finish = scan_data['finalized_at']

            # Ignore all scans that have not completed
            if status == 'completed':
                report = request_data('GET', '/was/v2/scans/' + was_scan_id + '/report')
                high = []
                medium = []
                low = []
                try:
                    name = report['config']['name']
                    target = report['config']['settings']['target']

                    for finding in report['findings']:
                        risk = finding['risk_factor']
                        plugin_id = finding['plugin_id']
                        if risk == 'high':
                            high.append(plugin_id)
                        elif risk == 'medium':
                            medium.append(plugin_id)
                        elif risk == 'low':
                            low.append(plugin_id)

                    print(textwrap.shorten(str(name), width=37).ljust(37), textwrap.shorten(str(target), width=40).ljust(40), str(len(high)).ljust(6), str(len(medium)).ljust(6), str(len(low)).ljust(6), str(start).ljust(25), str(finish))
                except TypeError:
                    pass
        print()



import click
import os


@click.command(help="Spin up a http server to extract data from the Docker container")
def http():
    try:
        os.system("python3 -m http.server")
    except os.error:
        print("This feature is for Docker container's only")



from .api_wrapper import request_data
from .error_msg import error_msg


def nessus_scanners():
    try:
        data = request_data('GET', '/scanners')
        print("\nScanner Name".ljust(36), "Scanner ID".ljust(20), "Scanner UUID")
        print("-" * 100)
        for scanners in data["scanners"]:
            print(str(scanners["name"].ljust(35)), str(scanners["id"]).ljust(20), str(scanners['uuid']))
        print()
    except Exception as E:
        error_msg(E)



import click
from .api_wrapper import request_data


@click.command(help="Resume a paused Scan")
@click.argument('scan_id')
def resume(scan_id):
    try:
        request_data('POST', '/scans/' + scan_id + '/resume')
    except:
        # Json error expected. Need to clean up api wrapper to fix this
        pass



import pprint
import click
from .api_wrapper import request_data
from .error_msg import error_msg


@click.command(help="Test the API ex: /scans ")
@click.argument('url')
def api(url):
    try:
        data = request_data('GET', url)
        pprint.pprint(data)

    except Exception as E:
        error_msg(E)



import csv
from .api_wrapper import request_data


def webapp_export():

    # Crete a csv file object
    with open('webapp_data.csv', mode='w') as csv_file:
        agent_writer = csv.writer(csv_file, delimiter=',', quotechar='"')
        # write our Header information first
        header_list = ["Hostname", "Critical", "High", "Medium", "Low", "Scan Note"]
        agent_writer.writerow(header_list)

        data = request_data('GET', '/scans')
        # cycle through all of the scans and pull out the webapp scan IDs

        for scans in data['scans']:
            csv_list = []
            if scans['type'] == 'webapp':
                scan_details = request_data('GET', '/scans/'+str(scans['id']))
                try:
                    hostname = scan_details['hosts'][0]['hostname']
                except:
                    hostname = " "
                try:
                    message = scan_details['notes'][0]['message']
                except:
                    message = " "
                try:
                    critical = scan_details['hosts'][0]['critical']
                except:
                    critical = 0
                try:
                    high = scan_details['hosts'][0]['high']
                except:
                    high = 0
                try:
                    medium = scan_details['hosts'][0]['medium']
                except:
                    medium = 0
                try:
                    low = scan_details['hosts'][0]['low']
                except:
                    low = 0

                if message != "Job expired while pending status.":
                    csv_list.append(hostname)

                    csv_list.append(critical)
                    csv_list.append(high)
                    csv_list.append(medium)
                    csv_list.append(low)
                    csv_list.append(message)
                    agent_writer.writerow(csv_list)



import time
from .api_wrapper import request_data


def scan_details(uuid):
    # pull the scan data
    details = request_data('GET', '/scans/' + str(uuid))

    print("\nThe Scanner name is : " + str(details["info"]['scanner_name']))
    print("\nThe Name of the scan is " + str(details["info"]["name"]))
    print("\nThe Scan ID is " + str(uuid))
    print("\nThe " + str(details["info"]["hostcount"]) + " host(s) that were scanned are below :\n")
    for hosts in details["hosts"]:
        print(hosts["hostname"])

    start_time = time.strftime("%a, %d %b %Y %H:%M:%S ", time.localtime(details["info"]["scan_start"]))
    print("\nscan start : " + start_time)
    try:
        stop = time.strftime("%a, %d %b %Y %H:%M:%S ", time.localtime(details["info"]["scan_end"]))
        print("scan finish : " + stop)

        duration = (details["info"]["scan_end"] - details["info"]["scan_start"]) / 60
        print("Duration : " + str(duration) + " Minutes")
    except KeyError:
        print("This scan is still running")
    print("Scan Notes Below : ")
    for notes in details["notes"]:
        print("         " + notes["title"])
        print("         " + notes["message"] + "\n")
    return



import click
import csv
from .database import new_db_connection
from .api_wrapper import request_data, request_delete
from .tag_helper import update_tag, confirm_tag_exists, return_tag_uuid
from sqlite3 import Error


@click.command(help="Create a Tag Category/Value Pair")
@click.option('--c', default='', help="Create a Tag with the following Category name")
@click.option('--v', default='', help="Create a Tag Value; requires --c and Category Name or UUID")
@click.option('--d', default='This Tag was created/updated by navi', help="Description for your Tag")
@click.option('--plugin', default='', help="Create a tag by plugin ID")
@click.option('--name', default='', help="Create a Tag by the text found in the Plugin Name")
@click.option('--group', default='', help="Create a Tag based on a Agent Group")
@click.option('--output', default='', help="Create a Tag based on the text in the output. Requires --plugin")
@click.option('--port', default='', help="Create a Tag based on Assets that have a port open.")
@click.option('--file', default='', help="Create a Tag based on IPs in a CSV file.")
@click.option('--scantime', default='', help="Create a Tag for assets that took longer than supplied minutes")
def tag(c, v, d, plugin, name, group, output, port, scantime, file):
    # Generator to split IPs into 2000 IP chunks
    def chunks(l, n):
        for i in range(0, len(l), n):
            yield l[i:i + n]

    # start a blank list; IP list is due to a bug
    tag_list = []
    ip_list = ""

    if c == '':
        print("Category is required.  Please use the --c command")
        exit()

    if v == '':
        print("Value is required. Please use the --v command")
        exit()

    if output != '' and plugin == '':
        print("You must supply a plugin")
        exit()

    if plugin:
        try:
            database = r"navi.db"
            conn = new_db_connection(database)
            with conn:
                cur = conn.cursor()
                # See if we want to refine our search by the output found in this plugin
                # this needs to have a JOIN statement to reduce the amount
                if output != "":
                    cur.execute("SELECT asset_ip, asset_uuid, output from vulns where plugin_id='" + plugin + "' and output LIKE '%" + output + "%';")
                else:
                    cur.execute("SELECT asset_ip, asset_uuid, output from vulns where plugin_id=%s;" % plugin)

                plugin_data = cur.fetchall()
                for x in plugin_data:
                    ip = x[0]
                    uuid = x[1]
                    # Check to make sure the UUID isn't already present, if it isn't add it to the list
                    if uuid not in tag_list:
                        tag_list.append(uuid)  # update functionality requires uuid.  Only using IP until API bug gets fixed
                        ip_list = ip_list + "," + ip
                    else:
                        pass
        except Error:
            pass

    if port != '':
        database = r"navi.db"
        conn = new_db_connection(database)
        with conn:
            cur = conn.cursor()
            # This should be reduced.  Use a Join statement and only pull the IP and uuid
            cur.execute("SELECT * from vulns where port=" + port + " and (plugin_id='11219' or plugin_id='14272' or plugin_id='14274' or plugin_id='34220' or plugin_id='10335');")

            data = cur.fetchall()

            try:
                for vulns in data:
                    ip = vulns[1]
                    uuid = vulns[2]
                    if uuid not in tag_list:
                        tag_list.append(uuid)
                        ip_list = ip_list + "," + ip
            except ValueError:
                pass

    if name != '':
        try:
            database = r"navi.db"
            conn = new_db_connection(database)
            with conn:
                cur = conn.cursor()
                cur.execute("SELECT asset_ip, asset_uuid, output from vulns where plugin_name LIKE '%" + name + "%';")

                plugin_data = cur.fetchall()
                for x in plugin_data:
                    ip = x[0]
                    uuid = x[1]
                    if uuid not in tag_list:
                        tag_list.append(uuid)
                        ip_list = ip_list + "," + ip
                    else:
                        pass
        except Error:
            pass

    if group != '':
        # Updating tags is only allowed via tenable ID(UUID); However you can't grab the UUID from the Agent URI
        # Need to research a better solution for this problem.  Need to submit a bug.  Going to just delete the tag for now.
        uuid_to_delete = return_tag_uuid(c, v)
        request_delete('DELETE', '/tags/values/' + str(uuid_to_delete))
        try:
            querystring = {"limit": "5000"}
            group_data = request_data('GET', '/scanners/1/agent-groups')
            for agent_group in group_data['groups']:
                group_name = agent_group['name']
                group_id = agent_group['id']

                if group_name == group:
                    data = request_data('GET', '/scanners/1/agent-groups/' + str(group_id) + '/agents', params=querystring)
                    ip_list = ''

                    for agent in data['agents']:
                        ip_address = agent['ip']
                        uuid = agent['uuid']
                        ip_list = ip_list + "," + ip_address
                        tag_list.append(uuid)
        except Error:
            print("You might not have agent groups, or you are using Nessus Manager.  ")

    if scantime != '':
        database = r"navi.db"
        conn = new_db_connection(database)
        with conn:
            cur = conn.cursor()
            cur.execute("SELECT asset_ip, asset_uuid, output from vulns where plugin_id='19506';")

            data = cur.fetchall()
            try:
                for vulns in data:

                    output = vulns[2]

                    # split the output by return
                    parsed_output = output.split("\n")

                    # grab the length so we can grab the seconds
                    length = len(parsed_output)

                    # grab the scan duration- second to the last variable
                    duration = parsed_output[length - 2]

                    # Split at the colon to grab the numerical value
                    seconds = duration.split(" : ")

                    # split to remove "secs"
                    number = seconds[1].split(" ")

                    # grab the number for our minute calculation
                    final_number = number[0]

                    # convert seconds into minutes
                    minutes = int(final_number) / 60

                    # grab assets that match the criteria
                    if minutes > int(scantime):
                        try:
                            ip_list = ip_list + "," + str(vulns[0])
                            tag_list.append(vulns[1])
                        except ValueError:
                            pass
                print()
            except ValueError:
                pass

    if file != '':
        with open(file, 'r', newline='') as new_file:
            add_ips = csv.reader(new_file)

            for row in add_ips:
                for ips in row:
                    # need to look grab UUIDS per IP for the ablity to update Tags
                    tag_list.append(ips)
                    ip_list = ip_list + "," + ips

    if ip_list == '':
        print("\nYour tag resulted in 0 Assets, therefore the tag wasn't created\n")
        exit()
    else:
        # Before updating confirm if the tag exists
        answer = confirm_tag_exists(c, v)

        if answer == 'yes':
            if len(tag_list) > 1999:
                # break the list into 2000 IP chunks
                for chunks in chunks(tag_list, 1999):
                    update_tag(c, v, chunks)
            else:
                update_tag(c, v, tag_list)
        else:
            if len(tag_list) > 1999:
                try:
                    payload = {"category_name": str(c), "value": str(v), "description": str(d)}
                    data = request_data('POST', '/tags/values', payload=payload)
                    value_uuid = data["uuid"]
                    cat_uuid = data['category_uuid']
                    print("\nI've created your new Tag - {} : {}\n".format(c, v))
                    print("The Category UUID is : {}\n".format(cat_uuid))
                    print("The Value UUID is : {}\n".format(value_uuid))
                    print("Your Tag list was over 2000 IPs.  Splitting the IPs into chunks and updating the tags now")
                    for chunks in chunks(tag_list, 1999):
                        update_tag(c, v, chunks)

                    print(str(len(tag_list)) + " IPs added to the Tag")
                except Exception as E:
                    print("Duplicate Tag Category: You may need to delete your tag first\n")
                    print("We could not confirm your tag name, is it named weird?\n")
                    print(E)
            else:

                try:
                    payload = {"category_name": str(c), "value": str(v), "description": str(d), "filters": {"asset": {"and": [{"field": "ipv4", "operator": "eq", "value": str(ip_list[1:])}]}}}
                    data = request_data('POST', '/tags/values', payload=payload)
                    try:
                        value_uuid = data["uuid"]
                        cat_uuid = data['category_uuid']
                        print("\nI've created your new Tag - {} : {}\n".format(c, v))
                        print("The Category UUID is : {}\n".format(cat_uuid))
                        print("The Value UUID is : {}\n".format(value_uuid))
                        print(str(len(tag_list)) + " IPs added to the Tag")
                    except Exception as E:
                        print("Duplicate Tag Category: You may need to delete your tag first\n")
                        print("We could not confirm your tag name, is it named weird?\n")
                        print(E)
                except:
                    print("Duplicate Category")



import click
from .api_wrapper import request_data


def get_scans_by_owner(owner):
    data = request_data('GET', '/scans')
    scan_data = []
    for scan in data['scans']:
        scan_id = scan['id']
        scan_owner = scan['owner']
        scan_name = scan['name']
        scan_uuid = scan['wizard_uuid']
        if scan_owner == owner:
            scan_data.append((scan_name, scan_id, owner, scan_uuid))
    return scan_data


def get_owner_uuid(owner):
    users = request_data('GET', '/users')
    user_uuid = 0
    for user in users['users']:
        if user['username'] == owner:
            user_uuid = user['id']
    return user_uuid


def get_targets(scan_id):
    scan_details = request_data('GET', '/scans/' + str(scan_id))
    try:
        targets = scan_details['info']['targets']
    except TypeError:
        targets = ""
    except KeyError:
        targets = ""

    try:
        tag_targets = scan_details['info']['tag_targets']
    except TypeError:
        tag_targets = ""

    scanner_name = scan_details['info']['scanner_name']

    return targets, tag_targets, scanner_name


def get_scanner_id(scan_name):
    scanners = request_data('GET', '/scanners')
    for scanner in scanners['scanners']:
        if scanner['name'] == scan_name:
            return scanner['id']


@click.command(help="Change Ownership")
@click.option('--owner', default='', help='Current Owner login.')
@click.option('--new', default='', help='New Owner login.')
@click.option('--who', default='', help="Check what scans a user owns")
@click.option('-v', is_flag=True, help="Verbose output")
def change(owner, new, who, v):

    if who:
        scans = get_scans_by_owner(who)

        print("\nScan Name".ljust(81), "Scan ID".ljust(10), "Scan Owner")
        print("-" * 150)
        print()
        for scan in scans:
            print(str(scan[0]).ljust(80), str(scan[1]).ljust(10), str(scan[2]))
        print()

    if owner:
        scans = get_scans_by_owner(owner)
        new_owner_uuid = get_owner_uuid(new)
        print("\n*** Scans that have not run, will produce HTTP 400 errors ***")
        print("\nYou Scans are being converted now.")
        print("\nThis can take some time")
        for scan in scans:
            scan_uuid = scan[3]
            scan_name = scan[0]
            scan_id = scan[1]
            targets, tag_targets, scanner_name = get_targets(scan_id)
            payload = dict(uuid=scan_uuid, settings={"name": scan_name,
                                                     "owner_id": new_owner_uuid,
                                                     "tag_targets": tag_targets,
                                                     "text_targets": targets,
                                                     "scanner_id": get_scanner_id(scanner_name)
                                                     })

            request_data('PUT', '/scans/' + str(scan_id), payload=payload)
            if v:
                print("\nDone - Payload: " + str(payload))



import click
import os


@click.command(help="Open up a Netcat listener to accept files over port 8000")
def listen():
    try:
        print("\nI'm opening a connection so you can send a file into the container")
        print("\nuse this command on your pc to send data to the connector: nc 0.0.0.0 8000 < \"yourfile.csv\"\n")
        os.system("nc -l -p 8000 > newfile.csv")
    except os.error:
        print("This command uses netcat and is only meant for navi running in a docker container")
        print("You probably don't have netcat installed")



import time
import threading
from queue import Queue
from sqlite3 import Error
from .api_wrapper import request_data
from .database import new_db_connection, drop_tables, insert_vulns
from .dbconfig import create_vulns_table

lock = threading.Lock()

q = Queue()

navi_id = 0


def worker():
    # The worker thread pulls an item from the queue and processes it
    while True:
        item = q.get()
        chunk_num = item[58:]
        # parse_data(request_data('GET', item))
        data = request_data('GET', item)
        parse_data(data, chunk_num)
        q.task_done()


def parse_data(chunk_data, chunk_number):
    database = r"navi.db"
    vuln_conn = new_db_connection(database)
    vuln_conn.execute('pragma journal_mode=wal;')
    with vuln_conn:
        try:
            # loop through all of the vulns in this chunk
            for vulns in chunk_data:
                # create a blank list to append asset details
                vuln_list = []
                global navi_id
                navi_id = navi_id + 1
                # Try block to ignore assets without IPs
                try:
                    vuln_list.append(navi_id)
                    try:
                        ipv4 = vulns['asset']['ipv4']
                        vuln_list.append(ipv4)
                    except KeyError:
                        vuln_list.append(" ")

                    try:
                        asset_uuid = vulns['asset']['uuid']
                        vuln_list.append(asset_uuid)
                    except KeyError:
                        vuln_list.append(" ")

                    try:
                        hostname = vulns['asset']['hostname']
                        vuln_list.append(hostname)
                    except KeyError:
                        vuln_list.append(" ")

                    try:
                        first_found = vulns['first_found']
                        vuln_list.append(first_found)
                    except KeyError:
                        vuln_list.append(" ")

                    try:
                        last_found = vulns['last_found']
                        vuln_list.append(last_found)
                    except KeyError:
                        vuln_list.append(" ")

                    try:
                        output = vulns['output']
                        vuln_list.append(output)
                    except KeyError:
                        vuln_list.append(" ")

                    try:
                        plugin_id = vulns['plugin']['id']
                        vuln_list.append(plugin_id)
                    except KeyError:
                        vuln_list.append(" ")

                    try:
                        plugin_name = vulns['plugin']['name']
                        vuln_list.append(plugin_name)
                    except KeyError:
                        vuln_list.append(" ")

                    try:
                        plugin_family = vulns['plugin']['family']
                        vuln_list.append(plugin_family)
                    except KeyError:
                        vuln_list.append(" ")
                    try:
                        port = vulns['port']['port']
                        vuln_list.append(port)
                    except KeyError:
                        vuln_list.append(" ")
                    try:
                        protocol = vulns['port']['protocol']
                        vuln_list.append(protocol)
                    except KeyError:
                        vuln_list.append(" ")

                    try:
                        severity = vulns['severity']
                        vuln_list.append(severity)
                    except KeyError:
                        vuln_list.append(" ")
                    try:
                        scan_completed = vulns['scan']['completed_at']
                        vuln_list.append(scan_completed)
                    except KeyError:
                        vuln_list.append(" ")

                    try:
                        scan_started = vulns['scan']['started_at']
                        vuln_list.append(scan_started)
                    except KeyError:
                        vuln_list.append(" ")

                    try:
                        scan_uuid = vulns['scan']['uuid']
                        vuln_list.append(scan_uuid)
                    except KeyError:
                        vuln_list.append(" ")

                    try:
                        schedule_id = vulns['scan']['schedule_id']
                        vuln_list.append(schedule_id)
                    except KeyError:
                        vuln_list.append(" ")

                    try:
                        state = vulns['state']
                        vuln_list.append(state)
                    except KeyError:
                        vuln_list.append(" ")
                    try:
                        insert_vulns(vuln_conn, vuln_list)
                    except Error as e:
                        print(e)
                except IndexError:
                    print("skipped one")
        except TypeError:
            print("Your Export has no data.  It may have expired")
            print("Error on Chunk " + chunk_number)

    print("Chunk " + chunk_number + " Finished")
    vuln_conn.close()


def vuln_export(days, ex_uuid, threads):
    start = time.time()
    # Crete a new connection to our database
    database = r"navi.db"
    drop_conn = new_db_connection(database)
    drop_conn.execute('pragma journal_mode=wal;')
    # Right now we just drop the table.  Eventually I will actually update the database
    drop_tables(drop_conn, 'vulns')

    create_vulns_table()

    # Set URLS for threading
    urls = []

    # Set the payload to the maximum number of assets to be pulled at once
    day = 86400
    new_limit = day * int(days)
    day_limit = time.time() - new_limit
    pay_load = {"num_assets": 500, "filters": {"last_found": int(day_limit)}}
    try:

        if ex_uuid == '0':
            # request an export of the data
            export = request_data('POST', '/vulns/export', payload=pay_load)

            # grab the export UUID
            ex_uuid = export['export_uuid']
            print('\nRequesting Vulnerability Export with ID : ' + ex_uuid)

            # set a variable to True for our While loop
            not_ready = True
        else:
            print("\nUsing your Export UUID\n")
            not_ready = True

        # now check the status
        status = request_data('GET', '/vulns/export/' + ex_uuid + '/status')

        # status = get_data('/vulns/export/89ac18d9-d6bc-4cef-9615-2d138f1ff6d2/status')
        print("Status : " + str(status["status"]))

        # loop to check status until finished
        while not_ready is True:
            # Pull the status, then pause 5 seconds and ask again.
            if status['status'] == 'PROCESSING' or 'QUEUED':
                time.sleep(15)
                status = request_data('GET', '/vulns/export/' + ex_uuid + '/status')
                print("Status : " + str(status["status"]))

            # Exit Loop once confirmed finished
            if status['status'] == 'FINISHED':
                ptime = time.time()
                print("\nProcessing Time took : " + str(ptime - start))

                # Display how many chunks there are
                avail = status['total_chunks']
                print("\nChunks Available - ", avail)
                print("Downloading chunks now...hold tight...This can take some time\n")
                not_ready = False

            # Tell the user an error occured
            if status['status'] == 'ERROR':
                print("Error occurred")

        # grab all of the chunks and craft the URLS for threading
        for y in status['chunks_available']:
            urls.append('/vulns/export/' + ex_uuid + '/chunks/' + str(y))

        for i in range(threads):
            t = threading.Thread(target=worker)
            t.daemon = True  # thread dies when main thread (only non-daemon thread) exits.
            t.start()

        for item in range(len(urls)):
            q.put(urls[item])

        q.join()
        end = time.time()
        print(end - start)

    except KeyError:
        print("Well this is a bummer; you don't have permissions to download Asset data :( ")

    except TypeError:
        print("You may not be authorized or your keys are invalid")
